@@ -20,11 +20,10 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 
-#include <iostream>
-#include <chrono>
-#include <string>
 #include <cstdio>
 #include <cstdlib>
+#include <iostream>
+#include <string>
 
 // Google Benchmark
 #include "benchmark/benchmark.h"
@@ -35,15 +34,6 @@
 
 #include "benchmark_utils.hpp"
 
-#define HIP_CHECK(condition)         \
-  {                                  \
-    hipError_t error = condition;    \
-    if(error != hipSuccess){         \
-        std::cout << "HIP error: " << error << " line: " << __LINE__ << std::endl; \
-        exit(error); \
-    } \
-  }
-
 enum memory_operation_method
 {
     block_primitives_transpose,
@@ -384,10 +374,17 @@ void run_benchmark(benchmark::State& state,
     }
     HIP_CHECK(hipDeviceSynchronize());
 
+    // HIP events creation
+    hipEvent_t start, stop;
+    HIP_CHECK(hipEventCreate(&start));
+    HIP_CHECK(hipEventCreate(&stop));
+
     const unsigned int batch_size = 10;
     for(auto _ : state)
     {
-        auto start = std::chrono::high_resolution_clock::now();
+        // Record start event
+        HIP_CHECK(hipEventRecord(start, stream));
+
         for(size_t i = 0; i < batch_size; i++)
         {
             hipLaunchKernelGGL(
@@ -396,14 +393,20 @@ void run_benchmark(benchmark::State& state,
                 d_input, d_output, selected_operation
             );
         }
-        HIP_CHECK(hipDeviceSynchronize());
-        HIP_CHECK(hipStreamSynchronize(stream));
 
-        auto end = std::chrono::high_resolution_clock::now();
-        auto elapsed_seconds =
-            std::chrono::duration_cast<std::chrono::duration<double>>(end - start);
-        state.SetIterationTime(elapsed_seconds.count());
+        // Record stop event and wait until it completes
+        HIP_CHECK(hipEventRecord(stop, stream));
+        HIP_CHECK(hipEventSynchronize(stop));
+
+        float elapsed_mseconds;
+        HIP_CHECK(hipEventElapsedTime(&elapsed_mseconds, start, stop));
+        state.SetIterationTime(elapsed_mseconds / 1000);
     }
+
+    // Destroy HIP events
+    HIP_CHECK(hipEventDestroy(start));
+    HIP_CHECK(hipEventDestroy(stop));
+
     state.SetBytesProcessed(state.iterations() * batch_size * size * sizeof(T));
     state.SetItemsProcessed(state.iterations() * batch_size * size);
 
@@ -440,22 +443,35 @@ void run_benchmark_memcpy(benchmark::State& state,
     }
     HIP_CHECK(hipDeviceSynchronize());
 
+    // HIP events creation
+    hipEvent_t start, stop;
+    HIP_CHECK(hipEventCreate(&start));
+    HIP_CHECK(hipEventCreate(&stop));
+
     const unsigned int batch_size = 10;
     for(auto _ : state)
     {
-        auto start = std::chrono::high_resolution_clock::now();
+        // Record start event
+        HIP_CHECK(hipEventRecord(start, stream));
+
         for(size_t i = 0; i < batch_size; i++)
         {
             HIP_CHECK(hipMemcpy(d_output, d_input, size * sizeof(T), hipMemcpyDeviceToDevice));
         }
-        HIP_CHECK(hipDeviceSynchronize());
-        HIP_CHECK(hipStreamSynchronize(stream));
 
-        auto end = std::chrono::high_resolution_clock::now();
-        auto elapsed_seconds =
-            std::chrono::duration_cast<std::chrono::duration<double>>(end - start);
-        state.SetIterationTime(elapsed_seconds.count());
+        // Record stop event and wait until it completes
+        HIP_CHECK(hipEventRecord(stop, stream));
+        HIP_CHECK(hipEventSynchronize(stop));
+
+        float elapsed_mseconds;
+        HIP_CHECK(hipEventElapsedTime(&elapsed_mseconds, start, stop));
+        state.SetIterationTime(elapsed_mseconds / 1000);
     }
+
+    // Destroy HIP events
+    HIP_CHECK(hipEventDestroy(start));
+    HIP_CHECK(hipEventDestroy(stop));
+
     state.SetBytesProcessed(state.iterations() * batch_size * size * sizeof(T));
     state.SetItemsProcessed(state.iterations() * batch_size * size);
 
@@ -463,17 +479,24 @@ void run_benchmark_memcpy(benchmark::State& state,
     HIP_CHECK(hipFree(d_output));
 }
 
-#define CREATE_BENCHMARK(METHOD, OPERATION, T, SIZE, BLOCK_SIZE, IPT) \
-benchmark::RegisterBenchmark( \
-    (#METHOD "_" #OPERATION "<" #T "," #SIZE ",BS:" #BLOCK_SIZE ",IPT:" #IPT ">"), \
-    run_benchmark<T, BLOCK_SIZE, IPT, METHOD, OPERATION \
-    >, SIZE, stream \
-)
+#define CREATE_BENCHMARK(METHOD, OPERATION, T, SIZE, BLOCK_SIZE, IPT)                     \
+    benchmark::RegisterBenchmark(                                                         \
+        bench_naming::format_name("{lvl:device,algo:memory,subalgo:" #METHOD              \
+                                  ",operation:" #OPERATION ",key_type:" #T ",size:" #SIZE \
+                                  ",cfg:{bs:" #BLOCK_SIZE ",ipt:" #IPT "}}")              \
+            .c_str(),                                                                     \
+        run_benchmark<T, BLOCK_SIZE, IPT, METHOD, OPERATION>,                             \
+        SIZE,                                                                             \
+        stream)
 
-#define CREATE_BENCHMARK_MEMCPY(T, SIZE) \
-benchmark::RegisterBenchmark( \
-    ("Memcpy<" #T "," #SIZE">"), run_benchmark_memcpy<T>, SIZE, stream \
-)
+#define CREATE_BENCHMARK_MEMCPY(T, SIZE)                                              \
+    benchmark::RegisterBenchmark(                                                     \
+        bench_naming::format_name("{lvl:device,algo:memory,subalgo:copy,key_type:" #T \
+                                  ",size:" #SIZE ",cfg:default_config}")              \
+            .c_str(),                                                                 \
+        run_benchmark_memcpy<T>,                                                      \
+        SIZE,                                                                         \
+        stream)
 
 template<class T>
 constexpr unsigned int megabytes(unsigned int size)
@@ -485,11 +508,16 @@ int main(int argc, char *argv[])
 {
     cli::Parser parser(argc, argv);
     parser.set_optional<int>("trials", "trials", -1, "number of iterations");
+    parser.set_optional<std::string>("name_format",
+                                     "name_format",
+                                     "human",
+                                     "either: json,human,txt");
     parser.run_and_exit_if_error();
 
     // Parse argv
     benchmark::Initialize(&argc, argv);
     const int trials = parser.get<int>("trials");
+    bench_naming::set_format(parser.get<std::string>("name_format"));
 
     // HIP
     hipStream_t stream = 0; // default
