@@ -21,11 +21,10 @@
 // SOFTWARE.
 
 #include <iostream>
-#include <chrono>
-#include <vector>
+#include <limits>
 #include <locale>
 #include <string>
-#include <limits>
+#include <vector>
 
 // Google Benchmark
 #include "benchmark/benchmark.h"
@@ -39,15 +38,6 @@
 // rocPRIM
 #include <rocprim/rocprim.hpp>
 
-#define HIP_CHECK(condition)         \
-  {                                   \
-    hipError_t error = condition;    \
-    if(error != hipSuccess){         \
-        std::cout << "HIP error: " << error << " line: " << __LINE__ << std::endl; \
-        exit(error); \
-    } \
-  }
-
 #ifndef DEFAULT_N
 const size_t DEFAULT_N = 1024 * 1024 * 32;
 #endif
@@ -174,9 +164,15 @@ void run_sort_keys_benchmark(benchmark::State& state,
     }
     HIP_CHECK(hipDeviceSynchronize());
 
+    // HIP events creation
+    hipEvent_t start, stop;
+    HIP_CHECK(hipEventCreate(&start));
+    HIP_CHECK(hipEventCreate(&stop));
+
     for (auto _ : state)
     {
-        auto start = std::chrono::high_resolution_clock::now();
+        // Record start event
+        HIP_CHECK(hipEventRecord(start, stream));
 
         for(size_t i = 0; i < batch_size; i++)
         {
@@ -190,13 +186,20 @@ void run_sort_keys_benchmark(benchmark::State& state,
                 )
             );
         }
-        HIP_CHECK(hipDeviceSynchronize());
 
-        auto end = std::chrono::high_resolution_clock::now();
-        auto elapsed_seconds =
-            std::chrono::duration_cast<std::chrono::duration<double>>(end - start);
-        state.SetIterationTime(elapsed_seconds.count());
+        // Record stop event and wait until it completes
+        HIP_CHECK(hipEventRecord(stop, stream));
+        HIP_CHECK(hipEventSynchronize(stop));
+
+        float elapsed_mseconds;
+        HIP_CHECK(hipEventElapsedTime(&elapsed_mseconds, start, stop));
+        state.SetIterationTime(elapsed_mseconds / 1000);
     }
+
+    // Destroy HIP events
+    HIP_CHECK(hipEventDestroy(start));
+    HIP_CHECK(hipEventDestroy(stop));
+
     state.SetBytesProcessed(state.iterations() * batch_size * size * sizeof(key_type));
     state.SetItemsProcessed(state.iterations() * batch_size * size);
 
@@ -333,9 +336,15 @@ void run_sort_pairs_benchmark(benchmark::State& state,
     }
     HIP_CHECK(hipDeviceSynchronize());
 
+    // HIP events creation
+    hipEvent_t start, stop;
+    HIP_CHECK(hipEventCreate(&start));
+    HIP_CHECK(hipEventCreate(&stop));
+
     for (auto _ : state)
     {
-        auto start = std::chrono::high_resolution_clock::now();
+        // Record start event
+        HIP_CHECK(hipEventRecord(start, stream));
 
         for(size_t i = 0; i < batch_size; i++)
         {
@@ -349,13 +358,20 @@ void run_sort_pairs_benchmark(benchmark::State& state,
                 )
             );
         }
-        HIP_CHECK(hipDeviceSynchronize());
 
-        auto end = std::chrono::high_resolution_clock::now();
-        auto elapsed_seconds =
-            std::chrono::duration_cast<std::chrono::duration<double>>(end - start);
-        state.SetIterationTime(elapsed_seconds.count());
+        // Record stop event and wait until it completes
+        HIP_CHECK(hipEventRecord(stop, stream));
+        HIP_CHECK(hipEventSynchronize(stop));
+
+        float elapsed_mseconds;
+        HIP_CHECK(hipEventElapsedTime(&elapsed_mseconds, start, stop));
+        state.SetIterationTime(elapsed_mseconds / 1000);
     }
+
+    // Destroy HIP events
+    HIP_CHECK(hipEventDestroy(start));
+    HIP_CHECK(hipEventDestroy(stop));
+
     state.SetBytesProcessed(
         state.iterations() * batch_size * size * (sizeof(key_type) + sizeof(value_type))
     );
@@ -376,7 +392,8 @@ void add_sort_keys_benchmarks(std::vector<benchmark::internal::Benchmark *> &ben
                               size_t min_size,
                               size_t target_size)
 {
-    std::string name = Traits<KeyT>::name();
+    std::string key_name   = Traits<KeyT>::name();
+    std::string value_name = Traits<rocprim::empty_type>::name();
     for(const auto segment_count : segment_counts)
     {
         for(const auto segment_length : segment_lengths)
@@ -386,13 +403,19 @@ void add_sort_keys_benchmarks(std::vector<benchmark::internal::Benchmark *> &ben
             {
                 continue;
             }
-            benchmarks.push_back(
-                benchmark::RegisterBenchmark(
-                    (std::string("sort_keys<") + name + ">(" + std::to_string(segment_count) +
-                        " segments with length ~" + std::to_string(segment_length) +")").c_str(),
-                    [=](benchmark::State &state) { run_sort_keys_benchmark<KeyT>(state, segment_count, segment_length, target_size, stream); }
-                )
-            );
+            benchmarks.push_back(benchmark::RegisterBenchmark(
+                bench_naming::format_name(
+                    "{lvl:device,algo:radix_sort_segmented,key_type:" + key_name + ",value_type:"
+                    + value_name + ",segment_count:" + std::to_string(segment_count)
+                    + ",segment_length:" + std::to_string(segment_length) + ",cfg:default_config}")
+                    .c_str(),
+                [=](benchmark::State& state) {
+                    run_sort_keys_benchmark<KeyT>(state,
+                                                  segment_count,
+                                                  segment_length,
+                                                  target_size,
+                                                  stream);
+                }));
         }
     }
 }
@@ -415,13 +438,20 @@ void add_sort_pairs_benchmarks(std::vector<benchmark::internal::Benchmark *> &be
             {
                 continue;
             }
-            benchmarks.push_back(
-                benchmark::RegisterBenchmark(
-                    (std::string("sort_pairs<") + key_name + ", " + value_name + ">(" + std::to_string(segment_count) +
-                        " segments with length ~" + std::to_string(segment_length) +")").c_str(),
-                    [=](benchmark::State &state) { run_sort_pairs_benchmark<KeyT, ValueT>(state, segment_count, segment_length, target_size, stream); }
-                )
-            );
+            benchmarks.push_back(benchmark::RegisterBenchmark(
+                bench_naming::format_name(
+                    "{lvl:device,algo:radix_sort_segmented,key_type:" + key_name + ",value_type:"
+                    + value_name + ",segment_count:" + std::to_string(segment_count)
+                    + ",segment_length:" + std::to_string(segment_length) + ",cfg:default_config}")
+                    .c_str(),
+                [=](benchmark::State& state)
+                {
+                    run_sort_pairs_benchmark<KeyT, ValueT>(state,
+                                                           segment_count,
+                                                           segment_length,
+                                                           target_size,
+                                                           stream);
+                }));
         }
     }
 }
@@ -431,12 +461,17 @@ int main(int argc, char *argv[])
     cli::Parser parser(argc, argv);
     parser.set_optional<size_t>("size", "size", DEFAULT_N, "number of values");
     parser.set_optional<int>("trials", "trials", -1, "number of iterations");
+    parser.set_optional<std::string>("name_format",
+                                     "name_format",
+                                     "human",
+                                     "either: json,human,txt");
     parser.run_and_exit_if_error();
 
     // Parse argv
     benchmark::Initialize(&argc, argv);
     const size_t size = parser.get<size_t>("size");
     const int trials = parser.get<int>("trials");
+    bench_naming::set_format(parser.get<std::string>("name_format"));
 
     // HIP
     hipStream_t stream = 0; // default
