@@ -20,14 +20,13 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 
-#include <iostream>
-#include <chrono>
-#include <vector>
-#include <locale>
 #include <codecvt>
-#include <string>
 #include <cstdio>
 #include <cstdlib>
+#include <iostream>
+#include <locale>
+#include <string>
+#include <vector>
 
 // Google Benchmark
 #include "benchmark/benchmark.h"
@@ -40,15 +39,6 @@
 
 #include <rocprim/rocprim.hpp>
 
-#define HIP_CHECK(condition)         \
-    {                                  \
-        hipError_t error = condition;    \
-        if(error != hipSuccess){         \
-            std::cout << "HIP error: " << error << " line: " << __LINE__ << std::endl; \
-            exit(error); \
-        } \
-    }
-
 #ifndef DEFAULT_N
 const size_t DEFAULT_N = 1024 * 1024 * 32;
 #endif
@@ -135,10 +125,17 @@ void run_flagged_benchmark(benchmark::State& state,
     }
     HIP_CHECK(hipDeviceSynchronize());
 
+    // HIP events creation
+    hipEvent_t start, stop;
+    HIP_CHECK(hipEventCreate(&start));
+    HIP_CHECK(hipEventCreate(&stop));
+
     const unsigned int batch_size = 10;
     for(auto _ : state)
     {
-        auto start = std::chrono::high_resolution_clock::now();
+        // Record start event
+        HIP_CHECK(hipEventRecord(start, stream));
+
         for(size_t i = 0; i < batch_size; i++)
         {
             rocprim::select(
@@ -152,13 +149,20 @@ void run_flagged_benchmark(benchmark::State& state,
                 stream
             );
         }
-        HIP_CHECK(hipDeviceSynchronize());
 
-        auto end = std::chrono::high_resolution_clock::now();
-        auto elapsed_seconds =
-            std::chrono::duration_cast<std::chrono::duration<double>>(end - start);
-        state.SetIterationTime(elapsed_seconds.count());
+        // Record stop event and wait until it completes
+        HIP_CHECK(hipEventRecord(stop, stream));
+        HIP_CHECK(hipEventSynchronize(stop));
+
+        float elapsed_mseconds;
+        HIP_CHECK(hipEventElapsedTime(&elapsed_mseconds, start, stop));
+        state.SetIterationTime(elapsed_mseconds / 1000);
     }
+
+    // Destroy HIP events
+    HIP_CHECK(hipEventDestroy(start));
+    HIP_CHECK(hipEventDestroy(stop));
+
     state.SetBytesProcessed(state.iterations() * batch_size * size * sizeof(T));
     state.SetItemsProcessed(state.iterations() * batch_size * size);
 
@@ -237,10 +241,17 @@ void run_selectop_benchmark(benchmark::State& state,
     }
     HIP_CHECK(hipDeviceSynchronize());
 
+    // HIP events creation
+    hipEvent_t start, stop;
+    HIP_CHECK(hipEventCreate(&start));
+    HIP_CHECK(hipEventCreate(&stop));
+
     const unsigned int batch_size = 10;
     for(auto _ : state)
     {
-        auto start = std::chrono::high_resolution_clock::now();
+        // Record start event
+        HIP_CHECK(hipEventRecord(start, stream));
+
         for(size_t i = 0; i < batch_size; i++)
         {
             rocprim::select(
@@ -254,13 +265,20 @@ void run_selectop_benchmark(benchmark::State& state,
                 stream
             );
         }
-        HIP_CHECK(hipDeviceSynchronize());
 
-        auto end = std::chrono::high_resolution_clock::now();
-        auto elapsed_seconds =
-            std::chrono::duration_cast<std::chrono::duration<double>>(end - start);
-        state.SetIterationTime(elapsed_seconds.count());
+        // Record stop event and wait until it completes
+        HIP_CHECK(hipEventRecord(stop, stream));
+        HIP_CHECK(hipEventSynchronize(stop));
+
+        float elapsed_mseconds;
+        HIP_CHECK(hipEventElapsedTime(&elapsed_mseconds, start, stop));
+        state.SetIterationTime(elapsed_mseconds / 1000);
     }
+
+    // Destroy HIP events
+    HIP_CHECK(hipEventDestroy(start));
+    HIP_CHECK(hipEventDestroy(stop));
+
     state.SetBytesProcessed(state.iterations() * batch_size * size * sizeof(T));
     state.SetItemsProcessed(state.iterations() * batch_size * size);
 
@@ -345,10 +363,17 @@ void run_unique_benchmark(benchmark::State& state,
     }
     HIP_CHECK(hipDeviceSynchronize());
 
+    // HIP events creation
+    hipEvent_t start, stop;
+    HIP_CHECK(hipEventCreate(&start));
+    HIP_CHECK(hipEventCreate(&stop));
+
     const unsigned int batch_size = 10;
     for(auto _ : state)
     {
-        auto start = std::chrono::high_resolution_clock::now();
+        // Record start event
+        HIP_CHECK(hipEventRecord(start, stream));
+
         for(size_t i = 0; i < batch_size; i++)
         {
             rocprim::unique(
@@ -362,13 +387,20 @@ void run_unique_benchmark(benchmark::State& state,
                 stream
             );
         }
-        HIP_CHECK(hipDeviceSynchronize());
 
-        auto end = std::chrono::high_resolution_clock::now();
-        auto elapsed_seconds =
-            std::chrono::duration_cast<std::chrono::duration<double>>(end - start);
-        state.SetIterationTime(elapsed_seconds.count());
+        // Record stop event and wait until it completes
+        HIP_CHECK(hipEventRecord(stop, stream));
+        HIP_CHECK(hipEventSynchronize(stop));
+
+        float elapsed_mseconds;
+        HIP_CHECK(hipEventElapsedTime(&elapsed_mseconds, start, stop));
+        state.SetIterationTime(elapsed_mseconds / 1000);
     }
+
+    // Destroy HIP events
+    HIP_CHECK(hipEventDestroy(start));
+    HIP_CHECK(hipEventDestroy(stop));
+
     state.SetBytesProcessed(state.iterations() * batch_size * size * sizeof(T));
     state.SetItemsProcessed(state.iterations() * batch_size * size);
 
@@ -458,10 +490,17 @@ void run_unique_by_key_benchmark(benchmark::State& state,
     }
     HIP_CHECK(hipDeviceSynchronize());
 
+    // HIP events creation
+    hipEvent_t start, stop;
+    HIP_CHECK(hipEventCreate(&start));
+    HIP_CHECK(hipEventCreate(&stop));
+
     const unsigned int batch_size = 10;
     for(auto _ : state)
     {
-        auto start = std::chrono::high_resolution_clock::now();
+        // Record start event
+        HIP_CHECK(hipEventRecord(start, stream));
+
         for(size_t i = 0; i < batch_size; i++)
         {
             rocprim::unique_by_key(d_temp_storage,
@@ -475,13 +514,20 @@ void run_unique_by_key_benchmark(benchmark::State& state,
                                    equality_op,
                                    stream);
         }
-        HIP_CHECK(hipDeviceSynchronize());
 
-        auto end = std::chrono::high_resolution_clock::now();
-        auto elapsed_seconds
-            = std::chrono::duration_cast<std::chrono::duration<double>>(end - start);
-        state.SetIterationTime(elapsed_seconds.count());
+        // Record stop event and wait until it completes
+        HIP_CHECK(hipEventRecord(stop, stream));
+        HIP_CHECK(hipEventSynchronize(stop));
+
+        float elapsed_mseconds;
+        HIP_CHECK(hipEventElapsedTime(&elapsed_mseconds, start, stop));
+        state.SetIterationTime(elapsed_mseconds / 1000);
     }
+
+    // Destroy HIP events
+    HIP_CHECK(hipEventDestroy(start));
+    HIP_CHECK(hipEventDestroy(stop));
+
     state.SetBytesProcessed(state.iterations() * batch_size * size * (sizeof(Key) + sizeof(Value)));
     state.SetItemsProcessed(state.iterations() * batch_size * size);
 
@@ -493,29 +539,46 @@ void run_unique_by_key_benchmark(benchmark::State& state,
     hipFree(d_temp_storage);
 }
 
-#define CREATE_SELECT_FLAGGED_BENCHMARK(T, F, p) \
-benchmark::RegisterBenchmark( \
-    ("select_flagged<" #T "," #F ", "#T", unsigned int>(p = " #p")"), \
-    run_flagged_benchmark<T, F>, size, stream, p \
-)
+#define CREATE_SELECT_FLAGGED_BENCHMARK(T, F, p)                                              \
+    benchmark::RegisterBenchmark(                                                             \
+        bench_naming::format_name("{lvl:device,algo:select,subalgo:flagged,key_type:" #T      \
+                                  ",flag_type:" #F ",probability:" #p ",cfg:default_config}") \
+            .c_str(),                                                                         \
+        run_flagged_benchmark<T, F>,                                                          \
+        size,                                                                                 \
+        stream,                                                                               \
+        p)
 
-#define CREATE_SELECT_IF_BENCHMARK(T, p) \
-benchmark::RegisterBenchmark( \
-    ("select_if<" #T ", "#T", unsigned int>(p = " #p")"), \
-    run_selectop_benchmark<T>, size, stream, p \
-)
+#define CREATE_SELECT_IF_BENCHMARK(T, p)                                            \
+    benchmark::RegisterBenchmark(                                                   \
+        bench_naming::format_name("{lvl:device,algo:select,subalgo:if,key_type:" #T \
+                                  ",probability:" #p ",cfg:default_config}")        \
+            .c_str(),                                                               \
+        run_selectop_benchmark<T>,                                                  \
+        size,                                                                       \
+        stream,                                                                     \
+        p)
 
-#define CREATE_UNIQUE_BENCHMARK(T, p) \
-benchmark::RegisterBenchmark( \
-    ("unique<" #T ", "#T", unsigned int>(p = " #p")"), \
-    run_unique_benchmark<T>, size, stream, p \
-)
+#define CREATE_UNIQUE_BENCHMARK(T, p)                                                   \
+    benchmark::RegisterBenchmark(                                                       \
+        bench_naming::format_name("{lvl:device,algo:unique,key_type:" #T ",value_type:" \
+                                  + std::string(Traits<rocprim::empty_type>::name())    \
+                                  + ",probability:" #p ",cfg:default_config}")          \
+            .c_str(),                                                                   \
+        run_unique_benchmark<T>,                                                        \
+        size,                                                                           \
+        stream,                                                                         \
+        p)
 
-#define CREATE_UNIQUE_BY_KEY_BENCHMARK(K, V, p) \
-benchmark::RegisterBenchmark( \
-    ("unique_by_key<" #K ", "#V", unsigned int>(p = " #p")"), \
-    run_unique_by_key_benchmark<K, V>, size, stream, p \
-)
+#define CREATE_UNIQUE_BY_KEY_BENCHMARK(K, V, p)                                                   \
+    benchmark::RegisterBenchmark(bench_naming::format_name("{lvl:device,algo:unique,key_type:" #K \
+                                                           ",value_type:" #V ",probability:" #p   \
+                                                           ",cfg:default_config}")                \
+                                     .c_str(),                                                    \
+                                 run_unique_by_key_benchmark<K, V>,                               \
+                                 size,                                                            \
+                                 stream,                                                          \
+                                 p)
 
 #define BENCHMARK_FLAGGED_TYPE(type, value) \
     CREATE_SELECT_FLAGGED_BENCHMARK(type, value, 0.05f), \
@@ -546,12 +609,17 @@ int main(int argc, char *argv[])
     cli::Parser parser(argc, argv);
     parser.set_optional<size_t>("size", "size", DEFAULT_N, "number of values");
     parser.set_optional<int>("trials", "trials", -1, "number of iterations");
+    parser.set_optional<std::string>("name_format",
+                                     "name_format",
+                                     "human",
+                                     "either: json,human,txt");
     parser.run_and_exit_if_error();
 
     // Parse argv
     benchmark::Initialize(&argc, argv);
     const size_t size = parser.get<size_t>("size");
     const int trials = parser.get<int>("trials");
+    bench_naming::set_format(parser.get<std::string>("name_format"));
 
     // HIP
     hipStream_t stream = 0; // default
@@ -589,7 +657,7 @@ int main(int argc, char *argv[])
         BENCHMARK_UNIQUE_TYPE(int8_t),
         BENCHMARK_UNIQUE_TYPE(rocprim::half),
         BENCHMARK_UNIQUE_TYPE(custom_int_double),
-        
+
         BENCHMARK_UNIQUE_BY_KEY_TYPE(int, int),
         BENCHMARK_UNIQUE_BY_KEY_TYPE(float, double),
         BENCHMARK_UNIQUE_BY_KEY_TYPE(double, custom_double2),
