@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2023 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -37,7 +37,7 @@
     #error "rocPRIM requires at least C++14"
 #endif
 
-#ifndef ROCPRIM_DEVICE
+#if !defined(ROCPRIM_DEVICE) || defined(DOXYGEN_DOCUMENTATION_BUILD)
     #define ROCPRIM_DEVICE __device__
     #define ROCPRIM_HOST __host__
     #define ROCPRIM_HOST_DEVICE __host__ __device__
@@ -45,7 +45,7 @@
     #ifdef WIN32
     #define ROCPRIM_KERNEL __global__ static
     #else
-    #define ROCPRIM_KERNEL __global__
+        #define ROCPRIM_KERNEL __global__
     #endif
     // TODO: These parameters should be tuned for NAVI in the close future.
     #ifndef ROCPRIM_DEFAULT_MAX_BLOCK_SIZE
@@ -54,13 +54,20 @@
     #ifndef ROCPRIM_DEFAULT_MIN_WARPS_PER_EU
         #define ROCPRIM_DEFAULT_MIN_WARPS_PER_EU 1
     #endif
-    // Currently HIP on Windows has a bug involving inline device functions generating
-    // local memory/register allocation errors during compilation.  Current workaround is to
-    // use __attribute__((always_inline)) for the affected functions
-    #ifdef WIN32
-      #define ROCPRIM_INLINE inline __attribute__((always_inline))
+
+    #ifndef DOXYGEN_DOCUMENTATION_BUILD
+        // Currently HIP on Windows has a bug involving inline device functions generating
+        // local memory/register allocation errors during compilation.  Current workaround is to
+        // use __attribute__((always_inline)) for the affected functions
+        #ifdef _WIN32
+            #define ROCPRIM_INLINE inline __attribute__((always_inline))
+        #else
+            #define ROCPRIM_INLINE inline
+        #endif
     #else
-      #define ROCPRIM_INLINE inline
+        // Prefer simpler signatures to let Sphinx/Breathe parse them
+        #define ROCPRIM_FORCE_INLINE inline
+        #define ROCPRIM_INLINE inline
     #endif
     #define ROCPRIM_FORCE_INLINE __attribute__((always_inline))
 #endif
@@ -69,12 +76,6 @@
     #define ROCPRIM_DETAIL_USE_DPP true
 #endif
 
-#ifdef ROCPRIM_DISABLE_LOOKBACK_SCAN
-    #define ROCPRIM_DETAIL_USE_LOOKBACK_SCAN false
-#else
-    #define ROCPRIM_DETAIL_USE_LOOKBACK_SCAN true
-#endif
-
 #ifndef ROCPRIM_THREAD_LOAD_USE_CACHE_MODIFIERS
     #define ROCPRIM_THREAD_LOAD_USE_CACHE_MODIFIERS 1
 #endif
