@@ -1,4 +1,4 @@
-// Copyright (c) 2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -71,6 +71,16 @@ ROCPRIM_DEVICE ROCPRIM_INLINE void
     lookback_scan_state.initialize_prefix(flat_thread_id, number_of_blocks);
 }
 
+template<typename LookBackScanState>
+ROCPRIM_DEVICE ROCPRIM_INLINE void init_lookback_scan_state(LookBackScanState  lookback_scan_state,
+                                                            const unsigned int number_of_blocks,
+                                                            unsigned int       flat_thread_id)
+{
+
+    // Initialize lookback scan status.
+    lookback_scan_state.initialize_prefix(flat_thread_id, number_of_blocks);
+}
+
 template<typename LookBackScanState>
 ROCPRIM_KERNEL
     __launch_bounds__(ROCPRIM_DEFAULT_MAX_BLOCK_SIZE) void init_lookback_scan_state_kernel(
@@ -99,6 +109,33 @@ ROCPRIM_KERNEL
     init_lookback_scan_state(lookback_scan_state, number_of_blocks, ordered_bid, flat_thread_id);
 }
 
+template<typename LookBackScanState>
+ROCPRIM_KERNEL
+    __launch_bounds__(ROCPRIM_DEFAULT_MAX_BLOCK_SIZE) void init_lookback_scan_state_kernel(
+        LookBackScanState                             lookback_scan_state,
+        const unsigned int                            number_of_blocks,
+        unsigned int                                  save_index = 0,
+        typename LookBackScanState::value_type* const save_dest  = nullptr)
+{
+    const unsigned int block_id        = ::rocprim::detail::block_id<0>();
+    const unsigned int block_size      = ::rocprim::detail::block_size<0>();
+    const unsigned int block_thread_id = ::rocprim::detail::block_thread_id<0>();
+    const unsigned int flat_thread_id  = (block_id * block_size) + block_thread_id;
+
+    // Save the reduction (i.e. the last prefix) from the previous user of lookback_scan_state.
+    if(save_dest != nullptr)
+    {
+        access_indexed_lookback_value(lookback_scan_state,
+                                      number_of_blocks,
+                                      save_index,
+                                      flat_thread_id,
+                                      [&](typename LookBackScanState::value_type value)
+                                      { *save_dest = value; });
+    }
+
+    init_lookback_scan_state(lookback_scan_state, number_of_blocks, flat_thread_id);
+}
+
     template <bool Exclusive,
               class BlockScan,
               class T,
@@ -181,4 +218,4 @@ ROCPRIM_KERNEL
 
 END_ROCPRIM_NAMESPACE
 
-#endif // ROCPRIM_DEVICE_SCAN_COMMON_HPP_
\ No newline at end of file
+#endif // ROCPRIM_DEVICE_SCAN_COMMON_HPP_
