@@ -31,6 +31,7 @@
 #include "../config.hpp"
 #include "../functional.hpp"
 
+#include "../detail/temp_storage.hpp"
 #include "../detail/various.hpp"
 #include "../iterator/counting_iterator.hpp"
 #include "../iterator/transform_iterator.hpp"
@@ -114,21 +115,17 @@ hipError_t adjacent_difference_impl(void* const          temporary_storage,
     static constexpr unsigned int items_per_block  = block_size * items_per_thread;
 
     const std::size_t num_blocks = ceiling_div(size, items_per_block);
+    const std::size_t num_previous_values = InPlace && num_blocks >= 2 ? num_blocks - 1 : 0;
 
-    if(temporary_storage == nullptr)
+    value_type* previous_values;
+
+    const hipError_t partition_result = detail::temp_storage::partition(
+        temporary_storage,
+        storage_size,
+        detail::temp_storage::ptr_aligned_array(&previous_values, num_previous_values));
+    if(partition_result != hipSuccess || temporary_storage == nullptr)
     {
-        if(InPlace && num_blocks >= 2)
-        {
-            storage_size = align_size((num_blocks - 1) * sizeof(value_type));
-        }
-        else
-        {
-            // Make sure user won't try to allocate 0 bytes memory, because
-            // hipMalloc will return nullptr when size is zero.
-            storage_size = 4;
-        }
-
-        return hipSuccess;
+        return partition_result;
     }
 
     if(num_blocks == 0)
@@ -137,8 +134,7 @@ hipError_t adjacent_difference_impl(void* const          temporary_storage,
     }
 
     // Copy values before they are overwritten to use as tile predecessors/successors
-    // this is not dereferenced when the operation is not in place
-    auto* const previous_values = static_cast<value_type*>(temporary_storage);
+    // previous_values is not dereferenced when the operation is not in place
     if ROCPRIM_IF_CONSTEXPR(InPlace)
     {
         // If doing left adjacent diff then the last item of each block is needed for the
