@@ -21,10 +21,12 @@
 #ifndef ROCPRIM_DEVICE_DEVICE_MERGE_HPP_
 #define ROCPRIM_DEVICE_DEVICE_MERGE_HPP_
 
-#include <type_traits>
+#include <iostream>
 #include <iterator>
+#include <type_traits>
 
 #include "../config.hpp"
+#include "../detail/temp_storage.hpp"
 #include "../detail/various.hpp"
 
 #include "device_merge_config.hpp"
@@ -147,14 +149,18 @@ hipError_t merge_impl(void * temporary_storage,
     static constexpr unsigned int items_per_thread = config::items_per_thread;
     static constexpr auto items_per_block = block_size * items_per_thread;
 
-    const unsigned int partitions = ((input1_size + input2_size) + items_per_block - 1) / items_per_block;
-    const size_t partition_bytes = (partitions + 1) * sizeof(unsigned int);
+    const unsigned int partitions
+        = ((input1_size + input2_size) + items_per_block - 1) / items_per_block;
 
-    if(temporary_storage == nullptr)
+    unsigned int* index;
+
+    const hipError_t partition_result = detail::temp_storage::partition(
+        temporary_storage,
+        storage_size,
+        detail::temp_storage::ptr_aligned_array(&index, partitions + 1));
+    if(partition_result != hipSuccess || temporary_storage == nullptr)
     {
-        // storage_size is never zero
-        storage_size = partition_bytes;
-        return hipSuccess;
+        return partition_result;
     }
 
     if( partitions == 0u )
@@ -171,8 +177,6 @@ hipError_t merge_impl(void * temporary_storage,
         std::cout << "items_per_block " << items_per_block << '\n';
     }
 
-    unsigned int * index = reinterpret_cast<unsigned int *>(temporary_storage);
-
     const unsigned partition_blocks = ((partitions + 1) + half_block - 1) / half_block;
 
     if(debug_synchronous) start = std::chrono::high_resolution_clock::now();
