@@ -1,4 +1,4 @@
-// Copyright (c) 2018-2020 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2018-2022 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,8 +21,9 @@
 #ifndef ROCPRIM_DEVICE_DEVICE_RUN_LENGTH_ENCODE_HPP_
 #define ROCPRIM_DEVICE_DEVICE_RUN_LENGTH_ENCODE_HPP_
 
-#include <type_traits>
+#include <iostream>
 #include <iterator>
+#include <type_traits>
 
 #include "../config.hpp"
 #include "../detail/various.hpp"
@@ -378,18 +379,13 @@ hipError_t run_length_encode_non_trivial_runs(void * temporary_storage,
 
     // Read count of all runs (including trivial runs)
     count_type all_runs_count;
-    // hipMemcpyWithStream is only supported on rocm 3.1 and above
-    #if HIP_VERSION_MAJOR >= 3
-    #if HIP_VERSION_MINOR >= 1 || HIP_VERSION_MAJOR >= 4
-    error = hipMemcpyWithStream(&all_runs_count, all_runs_count_tmp, sizeof(count_type), hipMemcpyDeviceToHost, stream);
-    if(error != hipSuccess) return error;
-    #else
-    error = hipMemcpyAsync(&all_runs_count, all_runs_count_tmp, sizeof(count_type), hipMemcpyDeviceToHost, stream);
-    if(error != hipSuccess) return error;
-    error = hipStreamSynchronize(stream);
-    #endif
-    #endif
-
+    error = detail::memcpy_and_sync(&all_runs_count,
+                                    all_runs_count_tmp,
+                                    sizeof(count_type),
+                                    hipMemcpyDeviceToHost,
+                                    stream);
+    if(error != hipSuccess)
+        return error;
 
     // Select non-trivial runs
     if(debug_synchronous) start = std::chrono::high_resolution_clock::now();
