@@ -1,4 +1,4 @@
-// Copyright (c) 2021-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2021-2023 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -25,11 +25,15 @@
 #include "../device_merge_sort.hpp"
 #include "device_radix_block_sort.hpp"
 
+#include <type_traits>
+
 BEGIN_ROCPRIM_NAMESPACE
 
 namespace detail
 {
 
+/// In device_radix_sort, we use this device_radix_sort_merge_sort specialization only
+/// for low input sizes (< 1M elements).
 template<class Config,
          bool Descending,
          class KeysInputIterator,
@@ -55,44 +59,50 @@ inline hipError_t radix_sort_merge_impl(
     using value_type = typename std::iterator_traits<ValuesInputIterator>::value_type;
     const unsigned int current_radix_bits = end_bit - bit;
 
-    static constexpr bool with_custom_config = !std::is_same<Config, default_config>::value;
+    constexpr bool is_default_config = std::is_same<Config, default_config>::value;
+    // no need to provide default values for merge_sort_config as it only
+    // contains two autotuned subalgorithms
 
-    // In device_radix_sort, we use this device_radix_sort_merge_sort specialization only
-    // for low input sizes (< 1M elements), so we hardcode a kernel configuration most
-    // suitable for this (maximum: <256u, 4u>).
-    // Use <256u, 4u>, unless smaller is needed to not exceed shared memory maximum.
-    using default_radix_sort_block_sort_config =
-        typename rocprim::detail::radix_sort_block_sort_config_base<key_type, value_type>::type;
-    using block_sort_config
-        = kernel_config<rocprim::min(256u, default_radix_sort_block_sort_config::block_size),
-                        rocprim::min(4u, default_radix_sort_block_sort_config::items_per_thread)>;
+    // In the case that the user provides no custom config for merge sort block sort,
+    // instead of using the autotuned merge_sort_block_sort_config, use a hard-coded config that
+    // a power-of-two items sorted per block.
+    using default_block_sort_config =
+        typename radix_sort_block_sort_config_base<key_type, value_type>::type;
+    using radix_sort_block_sort_config =
+        typename std::conditional<is_default_config,
+                                  default_block_sort_config,
+                                  // extract the relevant config from merge_sort_block_sort_config
+                                  typename Config::block_sort_config::sort_config>::type;
+    static_assert(
+        is_power_of_two(radix_sort_block_sort_config::block_size
+                        * radix_sort_block_sort_config::items_per_thread),
+        "The sorted items per block of the radix sort merge sort must be a power of two.");
 
-    using block_merge_config = typename std::
-        conditional<with_custom_config, typename Config::block_merge_config, default_config>::type;
+    using merge_sort_block_merge_config = typename std::
+        conditional<is_default_config, default_config, typename Config::block_merge_config>::type;
 
     // Wrap our radix_sort_block_sort kernel config in a merge_sort_block_sort_config
     // just so device_merge_sort_compile_time_verifier can check.
-    using wrapped_bs_config = wrapped_merge_sort_block_sort_config<
-        merge_sort_block_sort_config<block_sort_config::block_size,
-                                     block_sort_config::items_per_thread,
-                                     block_sort_algorithm::default_algorithm>,
-        key_type,
-        value_type>;
-    using wrapped_bm_config
-        = wrapped_merge_sort_block_merge_config<block_merge_config, key_type, value_type>;
+    using merge_sort_block_sort_config
+        = merge_sort_block_sort_config<radix_sort_block_sort_config::block_size,
+                                       radix_sort_block_sort_config::items_per_thread,
+                                       block_sort_algorithm::default_algorithm>;
+    using wrapped_bs_config
+        = wrapped_merge_sort_block_sort_config<merge_sort_block_sort_config, key_type, value_type>;
+    using wrapped_bm_config = wrapped_merge_sort_block_merge_config<merge_sort_block_merge_config,
+                                                                    key_type,
+                                                                    value_type>;
 
-    (void)device_merge_sort_compile_time_verifier<
-        wrapped_bs_config,
-        wrapped_bm_config>; // Some helpful checks during compile-time
+    // Some helpful checks during compile-time
+    (void)device_merge_sort_compile_time_verifier<wrapped_bs_config, wrapped_bm_config>;
 
+    // We will get this later from the block_sort algorithm
     unsigned int sort_items_per_block
-        = block_sort_config::block_size
-          * block_sort_config::
-              items_per_thread; // We will get this later from the block_sort algorithm
+        = radix_sort_block_sort_config::block_size * radix_sort_block_sort_config::items_per_thread;
 
     if(temporary_storage == nullptr)
     {
-        return merge_sort_block_merge<block_merge_config>(
+        return merge_sort_block_merge<merge_sort_block_merge_config>(
             temporary_storage,
             storage_size,
             keys_output,
@@ -111,22 +121,22 @@ inline hipError_t radix_sort_merge_impl(
         return hipSuccess;
     }
 
-    radix_sort_block_sort<block_sort_config, Descending>(keys_input,
-                                                         keys_output,
-                                                         values_input,
-                                                         values_output,
-                                                         size,
-                                                         sort_items_per_block,
-                                                         bit,
-                                                         end_bit,
-                                                         stream,
-                                                         debug_synchronous);
+    radix_sort_block_sort<radix_sort_block_sort_config, Descending>(keys_input,
+                                                                    keys_output,
+                                                                    values_input,
+                                                                    values_output,
+                                                                    size,
+                                                                    sort_items_per_block,
+                                                                    bit,
+                                                                    end_bit,
+                                                                    stream,
+                                                                    debug_synchronous);
     // ^ sort_items_per_block is now updated
     if(size > sort_items_per_block)
     {
         if(current_radix_bits == sizeof(key_type) * 8)
         {
-            return merge_sort_block_merge<block_merge_config>(
+            return merge_sort_block_merge<merge_sort_block_merge_config>(
                 temporary_storage,
                 storage_size,
                 keys_output,
@@ -141,7 +151,7 @@ inline hipError_t radix_sort_merge_impl(
         }
         else
         {
-            return merge_sort_block_merge<block_merge_config>(
+            return merge_sort_block_merge<merge_sort_block_merge_config>(
                 temporary_storage,
                 storage_size,
                 keys_output,
