@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2019 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -29,15 +29,6 @@
 // rocPRIM API
 #include <rocprim/rocprim.hpp>
 
-#define HIP_CHECK(condition)         \
-{                                    \
-    hipError_t error = condition;    \
-    if(error != hipSuccess){         \
-        std::cout << "HIP error: " << error << " line: " << __LINE__ << std::endl; \
-        exit(error); \
-    } \
-}
-
 unsigned int get_rocprim_version_on_device();
 
 #endif // ROCPRIM_TEST_DETAIL_GET_ROCPRIM_VERSION_HPP_
