@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2021 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -29,14 +29,15 @@ typed_test_suite_def(suite_name_array, name_suffix, block_params);
 typed_test_def(suite_name_single, name_suffix, InclusiveScan)
 {
     using T = typename TestFixture::input_type;
- // for bfloat16 and half we use double for host-side accumulation
+    // for bfloat16 and half we use double for host-side accumulation
     using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
     binary_op_type_host binary_op_host;
     using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
+    using cast_type             = typename test_utils::select_plus_operator_host<T>::cast_type;
     constexpr size_t block_size = TestFixture::block_size;
 
     int device_id = test_common_utils::obtain_device_from_ctest();
-    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
     // Given block size not supported
@@ -51,14 +52,14 @@ typed_test_def(suite_name_single, name_suffix, InclusiveScan)
     for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
     {
         unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
-        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+        SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
         // Generate data
         std::vector<T> output = test_utils::get_random_data<T>(size, 2, 50, seed_value);
         std::vector<T> output2 = output;
 
         // Calculate expected results on host
-        std::vector<T> expected(output.size(), (T)0);
+        std::vector<T> expected(output.size(), T(0));
         for(size_t i = 0; i < output.size() / block_size; i++)
         {
             acc_type accumulator(0);
@@ -66,7 +67,7 @@ typed_test_def(suite_name_single, name_suffix, InclusiveScan)
             {
                 auto idx = i * block_size + j;
                 accumulator = binary_op_host(output[idx], accumulator);
-                expected[idx] = static_cast<T>(accumulator);
+                expected[idx] = static_cast<cast_type>(accumulator);
             }
         }
 
@@ -96,10 +97,11 @@ typed_test_def(suite_name_single, name_suffix, InclusiveScanReduce)
     using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
     binary_op_type_host binary_op_host;
     using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
+    using cast_type             = typename test_utils::select_plus_operator_host<T>::cast_type;
     constexpr size_t block_size = TestFixture::block_size;
 
     int device_id = test_common_utils::obtain_device_from_ctest();
-    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
     // Given block size not supported
@@ -114,7 +116,7 @@ typed_test_def(suite_name_single, name_suffix, InclusiveScanReduce)
     for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
     {
         unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
-        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+        SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
         // Generate data
         std::vector<T> output = test_utils::get_random_data<T>(size, 2, 50, seed_value);
@@ -122,8 +124,8 @@ typed_test_def(suite_name_single, name_suffix, InclusiveScanReduce)
         std::vector<T> output_reductions(size / block_size);
 
         // Calculate expected results on host
-        std::vector<T> expected(output.size(), (T)0);
-        std::vector<T> expected_reductions(output_reductions.size(), (T)0);
+        std::vector<T> expected(output.size(), T(0));
+        std::vector<T> expected_reductions(output_reductions.size(), T(0));
         for(size_t i = 0; i < output.size() / block_size; i++)
         {
             acc_type accumulator(0);
@@ -131,7 +133,7 @@ typed_test_def(suite_name_single, name_suffix, InclusiveScanReduce)
             {
                 auto idx = i * block_size + j;
                 accumulator = binary_op_host(output[idx], accumulator);
-                expected[idx] = static_cast<T>(accumulator);
+                expected[idx] = static_cast<cast_type>(accumulator);
             }
             expected_reductions[i] = expected[(i+1) * block_size - 1];
         }
@@ -170,11 +172,12 @@ typed_test_def(suite_name_single, name_suffix, InclusiveScanPrefixCallback)
     using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
     binary_op_type_host binary_op_host;
     using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
+    using cast_type = typename test_utils::select_plus_operator_host<T>::cast_type;
 
     constexpr size_t block_size = TestFixture::block_size;
 
     int device_id = test_common_utils::obtain_device_from_ctest();
-    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
     // Given block size not supported
@@ -189,7 +192,7 @@ typed_test_def(suite_name_single, name_suffix, InclusiveScanPrefixCallback)
     for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
     {
         unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
-        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+        SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
         // Generate data
         std::vector<T> output = test_utils::get_random_data<T>(size, 2, 50, seed_value);
@@ -198,8 +201,8 @@ typed_test_def(suite_name_single, name_suffix, InclusiveScanPrefixCallback)
         T block_prefix = test_utils::get_random_value<T>(0, 5, seed_value);
 
         // Calculate expected results on host
-        std::vector<T> expected(output.size(), (T)0);
-        std::vector<T> expected_block_prefixes(output_block_prefixes.size(), (T)0);
+        std::vector<T> expected(output.size(), T(0));
+        std::vector<T> expected_block_prefixes(output_block_prefixes.size(), T(0));
         for(size_t i = 0; i < output.size() / block_size; i++)
         {
             acc_type accumulator = block_prefix;
@@ -207,7 +210,7 @@ typed_test_def(suite_name_single, name_suffix, InclusiveScanPrefixCallback)
             {
                 auto idx = i * block_size + j;
                 accumulator = binary_op_host(output[idx], accumulator);
-                expected[idx] = static_cast<T>(accumulator);
+                expected[idx] = static_cast<cast_type>(accumulator);
             }
             expected_block_prefixes[i] = expected[(i+1) * block_size - 1];
         }
@@ -246,11 +249,12 @@ typed_test_def(suite_name_single, name_suffix, ExclusiveScan)
     using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
     binary_op_type_host binary_op_host;
     using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
+    using cast_type = typename test_utils::select_plus_operator_host<T>::cast_type;
 
     constexpr size_t block_size = TestFixture::block_size;
 
     int device_id = test_common_utils::obtain_device_from_ctest();
-    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
     // Given block size not supported
@@ -265,7 +269,7 @@ typed_test_def(suite_name_single, name_suffix, ExclusiveScan)
     for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
     {
         unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
-        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+        SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
         // Generate data
         std::vector<T> output = test_utils::get_random_data<T>(size, 2, 50, seed_value);
@@ -273,7 +277,7 @@ typed_test_def(suite_name_single, name_suffix, ExclusiveScan)
         const T init = test_utils::get_random_value<T>(0, 5, seed_value);
 
         // Calculate expected results on host
-        std::vector<T> expected(output.size(), (T)0);
+        std::vector<T> expected(output.size(), T(0));
         for(size_t i = 0; i < output.size() / block_size; i++)
         {
             acc_type accumulator(init);
@@ -282,7 +286,7 @@ typed_test_def(suite_name_single, name_suffix, ExclusiveScan)
             {
                 auto idx = i * block_size + j;
                 accumulator = binary_op_host(output[idx-1], accumulator);
-                expected[idx] = static_cast<T>(accumulator);
+                expected[idx] = static_cast<cast_type>(accumulator);
             }
         }
 
@@ -312,11 +316,12 @@ typed_test_def(suite_name_single, name_suffix, ExclusiveScanReduce)
     using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
     binary_op_type_host binary_op_host;
     using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
+    using cast_type = typename test_utils::select_plus_operator_host<T>::cast_type;
 
     constexpr size_t block_size = TestFixture::block_size;
 
     int device_id = test_common_utils::obtain_device_from_ctest();
-    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
     if(block_size > test_utils::get_max_block_size())
@@ -330,7 +335,7 @@ typed_test_def(suite_name_single, name_suffix, ExclusiveScanReduce)
     for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
     {
         unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
-        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+        SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
         // Generate data
         std::vector<T> output = test_utils::get_random_data<T>(size, 2, 50, seed_value);
@@ -341,8 +346,8 @@ typed_test_def(suite_name_single, name_suffix, ExclusiveScanReduce)
         std::vector<T> output_reductions(size / block_size);
 
         // Calculate expected results on host
-        std::vector<T> expected(output.size(), (T)0);
-        std::vector<T> expected_reductions(output_reductions.size(), (T)0);
+        std::vector<T> expected(output.size(), T(0));
+        std::vector<T> expected_reductions(output_reductions.size(), T(0));
         for(size_t i = 0; i < output.size() / block_size; i++)
         {
             acc_type accumulator(init);
@@ -351,7 +356,7 @@ typed_test_def(suite_name_single, name_suffix, ExclusiveScanReduce)
             {
                 auto idx = i * block_size + j;
                 accumulator = binary_op_host(output[idx-1], accumulator);
-                expected[idx] = static_cast<T>(accumulator);
+                expected[idx] = static_cast<cast_type>(accumulator);
             }
             acc_type accumulator_reductions(0);
             expected_reductions[i] = 0;
@@ -359,7 +364,7 @@ typed_test_def(suite_name_single, name_suffix, ExclusiveScanReduce)
             {
                 auto idx = i * block_size + j;
                 accumulator_reductions = binary_op_host(accumulator_reductions, output[idx]);
-                expected_reductions[i] = static_cast<T>(accumulator_reductions);
+                expected_reductions[i] = static_cast<cast_type>(accumulator_reductions);
             }
         }
 
@@ -397,11 +402,12 @@ typed_test_def(suite_name_single, name_suffix, ExclusiveScanPrefixCallback)
     using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
     binary_op_type_host binary_op_host;
     using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
+    using cast_type = typename test_utils::select_plus_operator_host<T>::cast_type;
 
     constexpr size_t block_size = TestFixture::block_size;
 
     int device_id = test_common_utils::obtain_device_from_ctest();
-    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
     // Given block size not supported
@@ -416,7 +422,7 @@ typed_test_def(suite_name_single, name_suffix, ExclusiveScanPrefixCallback)
     for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
     {
         unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
-        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+        SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
         // Generate data
         std::vector<T> output = test_utils::get_random_data<T>(size, 2, 50, seed_value);
@@ -425,8 +431,8 @@ typed_test_def(suite_name_single, name_suffix, ExclusiveScanPrefixCallback)
         T block_prefix = test_utils::get_random_value<T>(0, 5, seed_value);
 
         // Calculate expected results on host
-        std::vector<T> expected(output.size(), (T)0);
-        std::vector<T> expected_block_prefixes(output_block_prefixes.size(), (T)0);
+        std::vector<T> expected(output.size(), T(0));
+        std::vector<T> expected_block_prefixes(output_block_prefixes.size(), T(0));
         for(size_t i = 0; i < output.size() / block_size; i++)
         {
             acc_type accumulator = block_prefix;
@@ -435,7 +441,7 @@ typed_test_def(suite_name_single, name_suffix, ExclusiveScanPrefixCallback)
             {
                 auto idx = i * block_size + j;
                 accumulator = binary_op_host(output[idx-1], accumulator);
-                expected[idx] = static_cast<T>(accumulator);
+                expected[idx] = static_cast<cast_type>(accumulator);
             }
 
             acc_type accumulator_block_prefixes = block_prefix;
@@ -444,7 +450,7 @@ typed_test_def(suite_name_single, name_suffix, ExclusiveScanPrefixCallback)
             {
                 auto idx = i * block_size + j;
                 accumulator_block_prefixes = binary_op_host(output[idx], accumulator_block_prefixes);
-                expected_block_prefixes[i] = static_cast<T>(accumulator_block_prefixes);
+                expected_block_prefixes[i] = static_cast<cast_type>(accumulator_block_prefixes);
             }
         }
 
