@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2023 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,6 +21,7 @@
 // SOFTWARE.
 
 #include "test_utils_sort_comparator.hpp"
+
 block_sort_test_suite_type_def(suite_name, name_suffix)
 
     typed_test_suite_def(suite_name, name_suffix, block_params);
@@ -103,19 +104,23 @@ void TestSortKeyValue()
                             output_value.size() * sizeof(value_type),
                             hipMemcpyHostToDevice));
 
-        // Running kernel
-        hipLaunchKernelGGL(HIP_KERNEL_NAME(sort_pairs_kernel<block_size,
-                                                             items_per_thread,
-                                                             key_type,
-                                                             value_type,
-                                                             algo,
-                                                             binary_op_type>),
-                           dim3(grid_size),
-                           dim3(block_size),
-                           0,
-                           stream,
-                           device_key_output,
-                           device_value_output);
+        // Running kernel, ignored if invalid size
+        if(size > 0)
+        {
+            hipLaunchKernelGGL(HIP_KERNEL_NAME(sort_pairs_kernel<block_size,
+                                                                 items_per_thread,
+                                                                 key_type,
+                                                                 value_type,
+                                                                 algo,
+                                                                 binary_op_type>),
+                               dim3(grid_size),
+                               dim3(block_size),
+                               0,
+                               stream,
+                               device_key_output,
+                               device_value_output);
+            HIP_CHECK(hipGetLastError());
+        }
 
         // Reading results back
         HIP_CHECK(hipMemcpy(output_key.data(),
@@ -219,15 +224,21 @@ void TestSortKey(std::vector<size_t> sizes)
                                 hipMemcpyHostToDevice));
 
             const unsigned int grid_size = rocprim::detail::ceiling_div(size, items_per_block);
-            // Running kernel
-            hipLaunchKernelGGL(
-                HIP_KERNEL_NAME(sort_keys_kernel<block_size, items_per_thread, key_type, algo>),
-                dim3(grid_size),
-                dim3(block_size),
-                0,
-                stream,
-                device_key_output,
-                size);
+            // Running kernel, ignored if invalid size
+            if(size > 0)
+            {
+                hipLaunchKernelGGL(HIP_KERNEL_NAME(sort_keys_kernel<block_size,
+                                                                    items_per_thread,
+                                                                    key_type*,
+                                                                    algo,
+                                                                    binary_op_type>),
+                                   dim3(grid_size),
+                                   dim3(block_size),
+                                   0,
+                                   stream,
+                                   device_key_output,
+                                   size);
+            }
 
             // Reading results back
             HIP_CHECK(hipMemcpy(output.data(),
@@ -241,6 +252,113 @@ void TestSortKey(std::vector<size_t> sizes)
         }
     }
 }
+
+struct less_tuple
+{
+    template<class A, class B>
+    ROCPRIM_HOST_DEVICE inline constexpr bool operator()(const rocprim::tuple<A, B>& a,
+                                                         const rocprim::tuple<A, B>& b) const
+    {
+        return rocprim::get<0>(a) < rocprim::get<0>(b);
+    }
+};
+
+template<unsigned int block_size, unsigned int items_per_thread, rocprim::block_sort_algorithm algo>
+void TestSortStableKey(std::vector<size_t> sizes)
+{
+    int device_id = test_common_utils::obtain_device_from_ctest();
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
+    HIP_CHECK(hipSetDevice(device_id));
+
+    using key_type       = int8_t;
+    using index_type     = unsigned int;
+    using binary_op_type = less_tuple;
+
+    static constexpr const unsigned int items_per_block = block_size * items_per_thread;
+    hipStream_t                         stream          = 0; // default
+
+    if(!is_buildable(block_size, items_per_thread, algo))
+    {
+        GTEST_SKIP();
+    }
+
+    for(size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
+    {
+        unsigned int seed_value
+            = seed_index < random_seeds_count ? rand() : seeds[seed_index - random_seeds_count];
+        SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
+
+        for(size_t size : sizes)
+        {
+            SCOPED_TRACE(testing::Message() << "with size = " << size);
+            if(size == 0 && test_common_utils::use_hmm())
+            {
+                // hipMallocManaged() currently doesnt support zero byte allocation
+                continue;
+            }
+
+            // Generate data
+            std::vector<key_type> keys
+                = test_utils::get_random_data<key_type>(size, -10, 10, seed_value);
+
+            std::vector<index_type> values;
+            std::iota(values.begin(), values.end(), 0u);
+            using tuple_type = rocprim::tuple<key_type, index_type>;
+            std::vector<tuple_type> tuples(size);
+            std::vector<tuple_type> expected(size);
+            for(size_t i = 0; i < expected.size(); i++)
+            {
+                tuples[i]   = rocprim::make_tuple(keys[i], values[i]);
+                expected[i] = rocprim::make_tuple(keys[i], values[i]);
+            }
+
+            // Preparing device
+            tuple_type* device_tuples_output;
+            HIP_CHECK(test_common_utils::hipMallocHelper(&device_tuples_output,
+                                                         tuples.size() * sizeof(tuple_type)));
+
+            HIP_CHECK(hipMemcpy(device_tuples_output,
+                                tuples.data(),
+                                tuples.size() * sizeof(tuple_type),
+                                hipMemcpyHostToDevice));
+
+            const unsigned int grid_size = rocprim::detail::ceiling_div(size, items_per_block);
+
+            // Running kernel
+            hipLaunchKernelGGL(HIP_KERNEL_NAME(sort_keys_kernel<block_size,
+                                                                items_per_thread,
+                                                                tuple_type*,
+                                                                algo,
+                                                                binary_op_type>),
+                               dim3(grid_size),
+                               dim3(block_size),
+                               0,
+                               stream,
+                               device_tuples_output,
+                               size);
+
+            // Reading results back
+            HIP_CHECK(hipMemcpy(tuples.data(),
+                                device_tuples_output,
+                                tuples.size() * sizeof(tuple_type),
+                                hipMemcpyDeviceToHost));
+
+            // Calculate expected results on host
+            binary_op_type binary_op;
+            for(size_t i = 0; i < rocprim::detail::ceiling_div(expected.size(), items_per_block);
+                i++)
+            {
+                std::stable_sort(expected.begin() + (i * items_per_block),
+                                 expected.begin() + std::min(size, (i + 1) * items_per_block),
+                                 binary_op);
+            }
+
+            test_utils::assert_eq(tuples, expected);
+
+            HIP_CHECK(hipFree(device_tuples_output));
+        }
+    }
+}
 #endif // TEST_ROCPRIM_TEST_BLOCK_SORT_HPP_
 // This file is included multiple times in the test_block_sort_[algo].cpp file, because
 // the test definitions below this header guard need to be compiled for each test suites:
@@ -282,6 +400,25 @@ typed_test_def(suite_name, name_suffix, SortKeysMultipleItemsPerThread)
     TestSortKey<block_size, items_per_thread, key_type, value_type, algo, binary_op_type>(sizes);
 }
 
+typed_test_def(suite_name, name_suffix, SortKeysStable)
+{
+    using key_type = typename TestFixture::key_type;
+    if(std::is_same<key_type, int>::value)
+    {
+        static constexpr const rocprim::block_sort_algorithm algo       = TEST_BLOCK_SORT_ALGORITHM;
+        static constexpr const unsigned int                  block_size = TestFixture::block_size;
+        static constexpr const unsigned int                  items_per_thread = 4;
+
+        std::vector<size_t> sizes = {1134 * items_per_thread * block_size};
+        TestSortStableKey<block_size, items_per_thread, algo>(sizes);
+        TestSortStableKey<block_size, 1, algo>(sizes);
+    }
+    else
+    {
+        GTEST_SKIP();
+    }
+}
+
 typed_test_def(suite_name, name_suffix, SortKeyValueMultipleItemsPerThread)
 {
     using key_type                                            = typename TestFixture::key_type;
@@ -301,7 +438,26 @@ typed_test_def(suite_name, name_suffix, SortKeyInputSizeNotMultipleOfBlockSize)
     static constexpr const rocprim::block_sort_algorithm algo = TEST_BLOCK_SORT_ALGORITHM;
     static constexpr const unsigned int                  block_size       = TestFixture::block_size;
     static constexpr const unsigned int                  items_per_thread = 1;
-    if(algo != rocprim::block_sort_algorithm::bitonic_sort || items_per_thread != 1u)
+    if(algo == rocprim::block_sort_algorithm::bitonic_sort && items_per_thread != 1u)
+    {
+        GTEST_SKIP();
+    }
+    std::vector<size_t> sizes
+        = {0, 53, 512, 5000, 34567, (1 << 17) - 1220, 1134 * 256, (1 << 20) - 123};
+    TestSortKey<block_size, items_per_thread, key_type, value_type, algo, binary_op_type>(sizes);
+}
+
+typed_test_def(suite_name,
+               name_suffix,
+               SortKeyInputSizeNotMultipleOfBlockSizeMultipleItemsPerThread)
+{
+    using key_type                                            = typename TestFixture::key_type;
+    using value_type                                          = typename TestFixture::value_type;
+    using binary_op_type                                      = typename rocprim::less<key_type>;
+    static constexpr const rocprim::block_sort_algorithm algo = TEST_BLOCK_SORT_ALGORITHM;
+    static constexpr const unsigned int                  block_size       = TestFixture::block_size;
+    static constexpr const unsigned int                  items_per_thread = 4;
+    if(algo != rocprim::block_sort_algorithm::stable_merge_sort)
     {
         GTEST_SKIP();
     }
