@@ -66,16 +66,16 @@ using custom_double2 = test_utils::custom_test_type<double>;
 
 typedef ::testing::Types<
     DeviceMergeParams<int, double>,
-    DeviceMergeParams<unsigned long, unsigned int, ::rocprim::greater<unsigned long> >,
+    DeviceMergeParams<unsigned long, unsigned int, rocprim::greater<unsigned long>>,
     DeviceMergeParams<float, custom_double2>,
     DeviceMergeParams<int, float>,
     DeviceMergeParams<int8_t, int8_t>,
     DeviceMergeParams<uint8_t, uint8_t>,
-    DeviceMergeParams<rocprim::half, rocprim::half, test_utils::half_less>,
-    DeviceMergeParams<rocprim::bfloat16, rocprim::bfloat16, test_utils::bfloat16_less>,
-    DeviceMergeParams<custom_double2, custom_int2, ::rocprim::greater<custom_double2> >,
-    DeviceMergeParams<custom_int2, char>
-> RocprimDeviceMergeTestsParams;
+    DeviceMergeParams<rocprim::half, rocprim::half, rocprim::less<rocprim::half>>,
+    DeviceMergeParams<rocprim::bfloat16, rocprim::bfloat16, rocprim::less<rocprim::bfloat16>>,
+    DeviceMergeParams<custom_double2, custom_int2, rocprim::greater<custom_double2>>,
+    DeviceMergeParams<custom_int2, char>>
+    RocprimDeviceMergeTestsParams;
 
 // size1, size2
 std::vector<std::tuple<size_t, size_t>> get_sizes()
@@ -103,7 +103,7 @@ TYPED_TEST_SUITE(RocprimDeviceMergeTests, RocprimDeviceMergeTestsParams);
 TYPED_TEST(RocprimDeviceMergeTests, MergeKey)
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
-    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
     using key_type = typename TestFixture::key_type;
@@ -133,7 +133,7 @@ TYPED_TEST(RocprimDeviceMergeTests, MergeKey)
         for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
         {
             unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
-            SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+            SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
             // Generate data
             std::vector<key_type> keys_input1 = test_utils::get_random_data<key_type>(size1, 0, size1, seed_value);
@@ -242,7 +242,7 @@ TYPED_TEST(RocprimDeviceMergeTests, MergeKey)
 TYPED_TEST(RocprimDeviceMergeTests, MergeKeyValue)
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
-    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
     using key_type = typename TestFixture::key_type;
@@ -275,7 +275,7 @@ TYPED_TEST(RocprimDeviceMergeTests, MergeKeyValue)
         for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
         {
             unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
-            SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+            SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
             // Generate data
             std::vector<key_type> keys_input1 = test_utils::get_random_data<key_type>(size1, 0, size1, seed_value);
