@@ -23,12 +23,13 @@
 #ifndef TEST_DEVICE_SEGMENTED_RADIX_SORT_HPP_
 #define TEST_DEVICE_SEGMENTED_RADIX_SORT_HPP_
 
-#include "common_test_header.hpp"
+#include "../common_test_header.hpp"
 
 // required rocprim headers
 #include <rocprim/device/device_segmented_radix_sort.hpp>
 
 // required test headers
+#include "test_utils_custom_float_type.hpp"
 #include "test_utils_sort_comparator.hpp"
 #include "test_utils_types.hpp"
 
@@ -100,27 +101,11 @@ public:
 
 TYPED_TEST_SUITE_P(RocprimDeviceSegmentedRadixSort);
 
-inline std::vector<size_t> get_sizes(int seed_value)
-{
-    std::vector<size_t> sizes = {
-        1024, 2048, 4096, 1792,
-        0, 1, 10, 53, 211, 500,
-        2345, 11001, 34567,
-        1000000,
-        (1 << 16) - 1220
-    };
-
-    const std::vector<size_t> random_sizes
-        = test_utils::get_random_data<size_t>(5, 1, 100000, seed_value);
-    sizes.insert(sizes.end(), random_sizes.begin(), random_sizes.end());
-    return sizes;
-}
-
 template<typename TestFixture>
 inline void sort_keys()
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
-    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
     using key_type                           = typename TestFixture::params::key_type;
@@ -146,17 +131,10 @@ inline void sort_keys()
     {
         unsigned int seed_value
             = seed_index < random_seeds_count ? rand() : seeds[seed_index - random_seeds_count];
-        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+        SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
-        const std::vector<size_t> sizes = get_sizes(seed_value);
-        for(size_t size : sizes)
+        for(size_t size : test_utils::get_sizes(seed_value))
         {
-            if(size == 0 && test_common_utils::use_hmm())
-            {
-                // hipMallocManaged() currently doesnt support zero byte allocation
-                continue;
-            }
-
             SCOPED_TRACE(testing::Message() << "with size = " << size);
 
             // Generate data
@@ -164,8 +142,8 @@ inline void sort_keys()
             if(rocprim::is_floating_point<key_type>::value)
             {
                 keys_input = test_utils::get_random_data<key_type>(size,
-                                                                   (key_type)-1000,
-                                                                   (key_type) + 1000,
+                                                                   static_cast<key_type>(-1000),
+                                                                   static_cast<key_type>(+1000),
                                                                    seed_value);
             }
             else
@@ -286,7 +264,7 @@ template<typename TestFixture>
 inline void sort_pairs()
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
-    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
     using key_type                    = typename TestFixture::params::key_type;
@@ -313,17 +291,10 @@ inline void sort_pairs()
     {
         unsigned int seed_value
             = seed_index < random_seeds_count ? rand() : seeds[seed_index - random_seeds_count];
-        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+        SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
-        const std::vector<size_t> sizes = get_sizes(seed_value);
-        for(size_t size : sizes)
+        for(size_t size : test_utils::get_sizes(seed_value))
         {
-            if(size == 0 && test_common_utils::use_hmm())
-            {
-                // hipMallocManaged() currently doesnt support zero byte allocation
-                continue;
-            }
-
             SCOPED_TRACE(testing::Message() << "with size = " << size);
 
             // Generate data
@@ -331,8 +302,8 @@ inline void sort_pairs()
             if(rocprim::is_floating_point<key_type>::value)
             {
                 keys_input = test_utils::get_random_data<key_type>(size,
-                                                                   (key_type)-1000,
-                                                                   (key_type) + 1000,
+                                                                   static_cast<key_type>(-1000),
+                                                                   static_cast<key_type>(+1000),
                                                                    seed_value);
             }
             else
@@ -498,7 +469,7 @@ template<typename TestFixture>
 inline void sort_keys_double_buffer()
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
-    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
     using key_type                    = typename TestFixture::params::key_type;
@@ -524,17 +495,10 @@ inline void sort_keys_double_buffer()
     {
         unsigned int seed_value
             = seed_index < random_seeds_count ? rand() : seeds[seed_index - random_seeds_count];
-        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+        SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
-        const std::vector<size_t> sizes = get_sizes(seed_value);
-        for(size_t size : sizes)
+        for(size_t size : test_utils::get_sizes(seed_value))
         {
-            if(size == 0 && test_common_utils::use_hmm())
-            {
-                // hipMallocManaged() currently doesnt support zero byte allocation
-                continue;
-            }
-
             SCOPED_TRACE(testing::Message() << "with size = " << size);
 
             // Generate data
@@ -542,8 +506,8 @@ inline void sort_keys_double_buffer()
             if(rocprim::is_floating_point<key_type>::value)
             {
                 keys_input = test_utils::get_random_data<key_type>(size,
-                                                                   (key_type)-1000,
-                                                                   (key_type) + 1000,
+                                                                   static_cast<key_type>(-1000),
+                                                                   static_cast<key_type>(+1000),
                                                                    seed_value);
             }
             else
@@ -663,7 +627,7 @@ template<typename TestFixture>
 inline void sort_pairs_double_buffer()
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
-    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
     using key_type                    = typename TestFixture::params::key_type;
@@ -690,17 +654,10 @@ inline void sort_pairs_double_buffer()
     {
         unsigned int seed_value
             = seed_index < random_seeds_count ? rand() : seeds[seed_index - random_seeds_count];
-        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+        SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
-        const std::vector<size_t> sizes = get_sizes(seed_value);
-        for(size_t size : sizes)
+        for(size_t size : test_utils::get_sizes(seed_value))
         {
-            if(size == 0 && test_common_utils::use_hmm())
-            {
-                // hipMallocManaged() currently doesnt support zero byte allocation
-                continue;
-            }
-
             SCOPED_TRACE(testing::Message() << "with size = " << size);
 
             // Generate data
@@ -708,8 +665,8 @@ inline void sort_pairs_double_buffer()
             if(rocprim::is_floating_point<key_type>::value)
             {
                 keys_input = test_utils::get_random_data<key_type>(size,
-                                                                   (key_type)-1000,
-                                                                   (key_type) + 1000,
+                                                                   static_cast<key_type>(-1000),
+                                                                   static_cast<key_type>(+1000),
                                                                    seed_value);
             }
             else
