@@ -20,7 +20,7 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 
-#include "common_test_header.hpp"
+#include "../common_test_header.hpp"
 
 // required rocprim headers
 #include <rocprim/device/device_select.hpp>
@@ -69,26 +69,12 @@ typedef ::testing::Types<
     DeviceSelectParams<test_utils::custom_test_type<double>, test_utils::custom_test_type<double>, int, true>
 > RocprimDeviceSelectTestsParams;
 
-std::vector<size_t> get_sizes(int seed_value)
-{
-    std::vector<size_t> sizes = {
-        2, 32, 64, 256,
-        1024, 2048,
-        3072, 4096,
-        27845, (1 << 18) + 1111
-    };
-    const std::vector<size_t> random_sizes = test_utils::get_random_data<size_t>(2, 1, 16384, seed_value);
-    sizes.insert(sizes.end(), random_sizes.begin(), random_sizes.end());
-    std::sort(sizes.begin(), sizes.end());
-    return sizes;
-}
-
 TYPED_TEST_SUITE(RocprimDeviceSelectTests, RocprimDeviceSelectTestsParams);
 
 TYPED_TEST(RocprimDeviceSelectTests, Flagged)
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
-    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
     using T = typename TestFixture::input_type;
@@ -102,10 +88,9 @@ TYPED_TEST(RocprimDeviceSelectTests, Flagged)
     for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
     {
         unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
-        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+        SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
-        const std::vector<size_t> sizes = get_sizes(seed_value);
-        for(auto size : sizes)
+        for(auto size : test_utils::get_sizes(seed_value))
         {
             SCOPED_TRACE(testing::Message() << "with size = " << size);
 
@@ -230,45 +215,14 @@ struct select_op
     __device__ __host__ inline
     bool operator()(const T& value) const
     {
-        if(value < T(50)) return true;
-        return false;
-    }
-};
-
-template<>
-struct select_op<rocprim::half>
-{
-    __device__ __host__ inline
-    bool operator()(const rocprim::half& value) const
-    {
-        #if __HIP_DEVICE_COMPILE__
-        if(value < rocprim::half(50)) return true;
-        #else
-        if(test_utils::half_less()(value, rocprim::half(50))) return true;
-        #endif
-        return false;
-    }
-};
-
-template<>
-struct select_op<rocprim::bfloat16>
-{
-    __device__ __host__ inline
-    bool operator()(const rocprim::bfloat16& value) const
-    {
-        #if __HIP_DEVICE_COMPILE__
-        if(value < rocprim::bfloat16(50)) return true;
-        #else
-        if(test_utils::bfloat16_less()(value, rocprim::bfloat16(50))) return true;
-        #endif
-        return false;
+        return rocprim::less<T>()(value, T(50));
     }
 };
 
 TYPED_TEST(RocprimDeviceSelectTests, SelectOp)
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
-    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
     using T = typename TestFixture::input_type;
@@ -281,10 +235,9 @@ TYPED_TEST(RocprimDeviceSelectTests, SelectOp)
     for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
     {
         unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
-        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+        SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
-        const std::vector<size_t> sizes = get_sizes(seed_value);
-        for(auto size : sizes)
+        for(auto size : test_utils::get_sizes(seed_value))
         {
             SCOPED_TRACE(testing::Message() << "with size = " << size);
 
@@ -400,82 +353,18 @@ std::vector<float> get_discontinuity_probabilities()
     return probabilities;
 }
 
-TYPED_TEST(RocprimDeviceSelectTests, UniqueEmptyInput)
-{
-    int device_id = test_common_utils::obtain_device_from_ctest();
-    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
-    HIP_CHECK(hipSetDevice(device_id));
-
-    using T = typename TestFixture::input_type;
-    using op_type = typename test_utils::select_equal_to_operator<T>::type;
-    const bool debug_synchronous = TestFixture::debug_synchronous;
-
-    hipStream_t stream = 0; // default stream
-
-    // Allocate and copy to device
-    unsigned int * d_selected_count_output;
-    HIP_CHECK(test_common_utils::hipMallocHelper(&d_selected_count_output, sizeof(unsigned int)));
-
-    size_t temp_storage_size_bytes;
-    // Get size of d_temp_storage
-    HIP_CHECK(
-        rocprim::unique(
-            nullptr,
-            temp_storage_size_bytes,
-            rocprim::make_constant_iterator<T>((T)123),
-            rocprim::make_discard_iterator(),
-            d_selected_count_output,
-            0,
-            op_type(),
-            stream,
-            debug_synchronous
-        )
-    );
-
-    void * d_temp_storage = nullptr;
-    HIP_CHECK(test_common_utils::hipMallocHelper(&d_temp_storage, temp_storage_size_bytes));
-
-    // Run
-    HIP_CHECK(
-        rocprim::unique(
-            d_temp_storage,
-            temp_storage_size_bytes,
-            rocprim::make_constant_iterator<T>((T)123),
-            rocprim::make_discard_iterator(),
-            d_selected_count_output,
-            0,
-            op_type(),
-            stream,
-            debug_synchronous
-        )
-    );
-    HIP_CHECK(hipDeviceSynchronize());
-
-    // Check if number of selected value is 0
-    unsigned int selected_count_output = 0;
-    HIP_CHECK(
-        hipMemcpy(
-            &selected_count_output, d_selected_count_output,
-            sizeof(unsigned int),
-            hipMemcpyDeviceToHost
-        )
-    );
-    ASSERT_EQ(selected_count_output, 0);
-
-    hipFree(d_selected_count_output);
-    hipFree(d_temp_storage);
-}
-
 TYPED_TEST(RocprimDeviceSelectTests, Unique)
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
-    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
     using T = typename TestFixture::input_type;
     using U = typename TestFixture::output_type;
-    using op_type = typename test_utils::select_equal_to_operator<T>::type;
+
+    using op_type      = rocprim::equal_to<T>;
     using scan_op_type = rocprim::plus<T>;
+
     static constexpr bool use_identity_iterator = TestFixture::use_identity_iterator;
     const bool debug_synchronous = TestFixture::debug_synchronous;
 
@@ -484,11 +373,10 @@ TYPED_TEST(RocprimDeviceSelectTests, Unique)
     for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
     {
         unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
-        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+        SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
-        const auto sizes = get_sizes(seed_value);
         const auto probabilities = get_discontinuity_probabilities();
-        for(auto size : sizes)
+        for(auto size : test_utils::get_sizes(seed_value))
         {
             SCOPED_TRACE(testing::Message() << "with size = " << size);
             for(auto p : probabilities)
@@ -523,12 +411,15 @@ TYPED_TEST(RocprimDeviceSelectTests, Unique)
                 // Calculate expected results on host
                 std::vector<U> expected;
                 expected.reserve(input.size());
-                expected.push_back(input[0]);
-                for(size_t i = 1; i < input.size(); i++)
+                if(size > 0)
                 {
-                    if(!op_type()(input[i-1], input[i]))
+                    expected.push_back(input[0]);
+                    for(size_t i = 1; i < input.size(); i++)
                     {
-                        expected.push_back(input[i]);
+                        if(!op_type()(input[i - 1], input[i]))
+                        {
+                            expected.push_back(input[i]);
+                        }
                     }
                 }
 
@@ -632,7 +523,7 @@ struct element_equal_operator
 TEST(RocprimDeviceSelectTests, UniqueGuardedOperator)
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
-    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
     using T = int64_t;
@@ -647,11 +538,10 @@ TEST(RocprimDeviceSelectTests, UniqueGuardedOperator)
     for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
     {
         unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
-        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+        SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
-        const auto sizes = get_sizes(seed_value);
         const auto probabilities = get_discontinuity_probabilities();
-        for(auto size : sizes)
+        for(auto size : test_utils::get_sizes(seed_value))
         {
             SCOPED_TRACE(testing::Message() << "with size = " << size);
             for(auto p : probabilities)
@@ -669,7 +559,6 @@ TEST(RocprimDeviceSelectTests, UniqueGuardedOperator)
                     );
                 }
 
-
                 // Allocate and copy to device
                 T * d_input;
                 F * d_flag;
@@ -700,12 +589,15 @@ TEST(RocprimDeviceSelectTests, UniqueGuardedOperator)
                 // Calculate expected results on host
                 std::vector<U> expected;
                 expected.reserve(input.size());
-                expected.push_back(input[0]);
-                for(size_t i = 1; i < input.size(); i++)
+                if(size > 0)
                 {
-                    if(!host_equal_op(input[i-1], input[i]))
+                    expected.push_back(input[0]);
+                    for(size_t i = 1; i < input.size(); i++)
                     {
-                        expected.push_back(input[i]);
+                        if(!host_equal_op(input[i - 1], input[i]))
+                        {
+                            expected.push_back(input[i]);
+                        }
                     }
                 }
 
@@ -830,7 +722,7 @@ TYPED_TEST_SUITE(RocprimDeviceUniqueByKeyTests, RocprimDeviceUniqueByKeyTestPara
 TYPED_TEST(RocprimDeviceUniqueByKeyTests, UniqueByKey)
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
-    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
     using key_type = typename TestFixture::key_type;
@@ -838,7 +730,7 @@ TYPED_TEST(RocprimDeviceUniqueByKeyTests, UniqueByKey)
     using output_key_type = typename TestFixture::output_key_type;
     using output_value_type = typename TestFixture::output_value_type;
 
-    using op_type = typename test_utils::select_equal_to_operator<key_type>::type;
+    using op_type = rocprim::equal_to<key_type>;
 
     using scan_op_type = rocprim::plus<key_type>;
     static constexpr bool use_identity_iterator = TestFixture::use_identity_iterator;
@@ -849,11 +741,10 @@ TYPED_TEST(RocprimDeviceUniqueByKeyTests, UniqueByKey)
     for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
     {
         unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
-        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+        SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
-        const auto sizes = get_sizes(seed_value);
         const auto probabilities = get_discontinuity_probabilities();
-        for(auto size : sizes)
+        for(auto size : test_utils::get_sizes(seed_value))
         {
             SCOPED_TRACE(testing::Message() << "with size = " << size);
             for(auto p : probabilities)
@@ -903,14 +794,17 @@ TYPED_TEST(RocprimDeviceUniqueByKeyTests, UniqueByKey)
                 std::vector<output_value_type> expected_values;
                 expected_keys.reserve(input_keys.size());
                 expected_values.reserve(input_values.size());
-                expected_keys.push_back(input_keys[0]);
-                expected_values.push_back(input_values[0]);
-                for(size_t i = 1; i < input_keys.size(); i++)
+                if(size > 0)
                 {
-                    if(!op_type()(input_keys[i-1], input_keys[i]))
+                    expected_keys.push_back(input_keys[0]);
+                    expected_values.push_back(input_values[0]);
+                    for(size_t i = 1; i < input_keys.size(); i++)
                     {
-                        expected_keys.push_back(input_keys[i]);
-                        expected_values.push_back(input_values[i]);
+                        if(!op_type()(input_keys[i - 1], input_keys[i]))
+                        {
+                            expected_keys.push_back(input_keys[i]);
+                            expected_values.push_back(input_values[i]);
+                        }
                     }
                 }
 
@@ -1013,25 +907,10 @@ INSTANTIATE_TEST_SUITE_P(RocprimDeviceSelectLargeInputFlaggedTest, RocprimDevice
     2048, 9643, 32768, 38713
 ));
 
-std::vector<size_t> get_large_sizes()
-{
-    std::vector<size_t> sizes = {
-        (size_t{1} << 30) - 1, size_t{1} << 30,
-        (size_t{1} << 31) - 1, size_t{1} << 31,
-        (size_t{1} << 32) - 1, size_t{1} << 32,
-        (size_t{1} << 35) - 1
-    };
-    const std::vector<size_t> random_sizes = test_utils::get_random_data<size_t>(
-        2, (size_t {1} << 30) + 1, (size_t {1} << 35) - 2, 0);
-    sizes.insert(sizes.end(), random_sizes.begin(), random_sizes.end());
-    std::sort(sizes.begin(), sizes.end());
-    return sizes;
-}
-
 TEST_P(RocprimDeviceSelectLargeInputTests, LargeInputFlagged)
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
-    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
     unsigned int flag_selector = GetParam();
@@ -1042,10 +921,11 @@ TEST_P(RocprimDeviceSelectLargeInputTests, LargeInputFlagged)
 
     hipStream_t stream = 0; // default stream
 
-    const std::vector<size_t> sizes = get_large_sizes();
-
-    for(auto size : sizes)
+    for(auto size : test_utils::get_large_sizes(0))
     {
+        // otherwise test is too long
+        if(size > (size_t{1} << 35))
+            break;
         SCOPED_TRACE(testing::Message() << "with size = " << size);
 
         // Generate data
@@ -1147,3 +1027,84 @@ TEST_P(RocprimDeviceSelectLargeInputTests, LargeInputFlagged)
         hipFree(d_temp_storage);
     }
 }
+
+TEST_P(RocprimDeviceSelectLargeInputTests, LargeInputUnique)
+{
+    static constexpr bool        debug_synchronous = false;
+    static constexpr hipStream_t stream            = 0;
+
+    const int device_id = test_common_utils::obtain_device_from_ctest();
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
+    HIP_CHECK(hipSetDevice(device_id));
+
+    const unsigned int segment_length = GetParam();
+
+    for(const auto size : test_utils::get_large_sizes(0))
+    {
+        // otherwise test is too long
+        if(size > (size_t{1} << 35))
+            break;
+        SCOPED_TRACE(testing::Message() << "with size = " << size);
+
+        auto input_it = rocprim::make_transform_iterator(rocprim::make_counting_iterator(size_t(0)),
+                                                         [segment_length](size_t i)
+                                                         { return i / segment_length; });
+
+        const size_t expected_output_size = rocprim::detail::ceiling_div(size, segment_length);
+        std::vector<size_t> expected_output(expected_output_size);
+        std::iota(expected_output.begin(), expected_output.end(), 0);
+
+        size_t* d_output{};
+        size_t* d_unique_count_output{};
+        HIP_CHECK(test_common_utils::hipMallocHelper(&d_output,
+                                                     sizeof(*d_output) * expected_output_size));
+        HIP_CHECK(test_common_utils::hipMallocHelper(&d_unique_count_output,
+                                                     sizeof(*d_unique_count_output)));
+
+        size_t temp_storage_size_bytes{};
+        void*  d_temp_storage{};
+        HIP_CHECK(rocprim::unique(d_temp_storage,
+                                  temp_storage_size_bytes,
+                                  input_it,
+                                  d_output,
+                                  d_unique_count_output,
+                                  size,
+                                  rocprim::equal_to<size_t>{},
+                                  stream,
+                                  debug_synchronous));
+        ASSERT_GT(temp_storage_size_bytes, 0);
+        HIP_CHECK(test_common_utils::hipMallocHelper(&d_temp_storage, temp_storage_size_bytes));
+
+        HIP_CHECK(rocprim::unique(d_temp_storage,
+                                  temp_storage_size_bytes,
+                                  input_it,
+                                  d_output,
+                                  d_unique_count_output,
+                                  size,
+                                  rocprim::equal_to<size_t>{},
+                                  stream,
+                                  debug_synchronous));
+
+        size_t unique_count_output{};
+        HIP_CHECK(hipMemcpyWithStream(&unique_count_output,
+                                      d_unique_count_output,
+                                      sizeof(unique_count_output),
+                                      hipMemcpyDeviceToHost,
+                                      stream));
+        ASSERT_EQ(unique_count_output, expected_output_size);
+
+        std::vector<size_t> output(expected_output_size);
+        HIP_CHECK(hipMemcpyWithStream(output.data(),
+                                      d_output,
+                                      sizeof(output[0]) * expected_output_size,
+                                      hipMemcpyDeviceToHost,
+                                      stream));
+
+        ASSERT_NO_FATAL_FAILURE(
+            test_utils::assert_eq(output, expected_output, expected_output.size()));
+
+        HIP_CHECK(hipFree(d_output));
+        HIP_CHECK(hipFree(d_unique_count_output));
+        HIP_CHECK(hipFree(d_temp_storage));
+    }
+}
