@@ -70,8 +70,18 @@ TYPED_TEST(RocprimTextureCacheIteratorTests, Transform)
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
     SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
+
+    hipDeviceProp_t props;
+    HIP_CHECK(hipGetDeviceProperties(&props, device_id));
+    std::string deviceName = std::string(props.gcnArchName);
+    if (deviceName.rfind("gfx94", 0) == 0) {
+        // This is a gfx94x device, so skip this test
+        SCOPED_TRACE(testing::Message() << "Skipping texture cache text for " << deviceName);
+        return;
+    }
+
     HIP_CHECK(hipSetDevice(device_id));
-    
+
     using T = typename TestFixture::input_type;
     using Iterator = typename rocprim::texture_cache_iterator<T>;
     const bool debug_synchronous = TestFixture::debug_synchronous;
