@@ -27,7 +27,7 @@ typed_test_suite_def(RocprimWarpScanTests, name_suffix, warp_params);
 typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScan)
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
-    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
     using T = typename TestFixture::params::type;
@@ -35,6 +35,7 @@ typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScan)
     using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
     binary_op_type_host binary_op_host;
     using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
+    using cast_type = typename test_utils::select_plus_operator_host<T>::cast_type;
 
     // logical warp side for warp primitive, execution warp size is always rocprim::warp_size()
     static constexpr size_t logical_warp_size = TestFixture::params::warp_size;
@@ -73,12 +74,12 @@ typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScan)
     for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
     {
         unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
-        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+        SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
         // Generate data
         std::vector<T> input = test_utils::get_random_data<T>(size, 2, 50, seed_value);
         std::vector<T> output(size);
-        std::vector<T> expected(output.size(), (T)0);
+        std::vector<T> expected(output.size(), T(0));
 
         // Calculate expected results on host
         for(size_t i = 0; i < input.size() / logical_warp_size; i++)
@@ -88,7 +89,7 @@ typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScan)
             {
                 auto idx = i * logical_warp_size + j;
                 accumulator = binary_op_host(input[idx], accumulator);
-                expected[idx] = static_cast<T>(accumulator);
+                expected[idx] = static_cast<cast_type>(accumulator);
             }
         }
 
@@ -137,7 +138,7 @@ typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScan)
         );
 
         // Validating results
-        test_utils::assert_near(output, expected, test_utils::precision_threshold<T>::percentage);
+        test_utils::assert_near(output, expected, test_utils::precision<T> * logical_warp_size);
 
         HIP_CHECK(hipFree(device_input));
         HIP_CHECK(hipFree(device_output));
@@ -148,7 +149,7 @@ typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScan)
 typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScanReduce)
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
-    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
     using T = typename TestFixture::params::type;
@@ -156,6 +157,7 @@ typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScanReduce)
     using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
     binary_op_type_host binary_op_host;
     using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
+    using cast_type = typename test_utils::select_plus_operator_host<T>::cast_type;
 
     // logical warp side for warp primitive, execution warp size is always rocprim::warp_size()
     static constexpr size_t logical_warp_size = TestFixture::params::warp_size;
@@ -194,14 +196,14 @@ typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScanReduce)
     for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
     {
         unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
-        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+        SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
         // Generate data
         std::vector<T> input = test_utils::get_random_data<T>(size, 2, 50, seed_value);
         std::vector<T> output(size);
         std::vector<T> output_reductions(size / logical_warp_size);
-        std::vector<T> expected(output.size(), (T)0);
-        std::vector<T> expected_reductions(output_reductions.size(), (T)0);
+        std::vector<T> expected(output.size(), T(0));
+        std::vector<T> expected_reductions(output_reductions.size(), T(0));
 
         // Calculate expected results on host
         for(size_t i = 0; i < output.size() / logical_warp_size; i++)
@@ -211,7 +213,7 @@ typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScanReduce)
             {
                 auto idx = i * logical_warp_size + j;
                 accumulator = binary_op_host(input[idx],accumulator);
-                expected[idx] = static_cast<T>(accumulator);
+                expected[idx] = static_cast<cast_type>(accumulator);
             }
             expected_reductions[i] = expected[(i+1) * logical_warp_size - 1];
         }
@@ -276,8 +278,10 @@ typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScanReduce)
         );
 
         // Validating results
-        test_utils::assert_near(output, expected, test_utils::precision_threshold<T>::percentage);
-        test_utils::assert_near(output_reductions, expected_reductions, test_utils::precision_threshold<T>::percentage);
+        test_utils::assert_near(output, expected, test_utils::precision<T> * logical_warp_size);
+        test_utils::assert_near(output_reductions,
+                                expected_reductions,
+                                test_utils::precision<T> * logical_warp_size);
 
         HIP_CHECK(hipFree(device_input));
         HIP_CHECK(hipFree(device_output));
@@ -289,14 +293,15 @@ typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScanReduce)
 typed_test_def(RocprimWarpScanTests, name_suffix, ExclusiveScan)
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
-    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
     using T = typename TestFixture::params::type;
-     // for bfloat16 and half we use double for host-side accumulation
+    // for bfloat16 and half we use double for host-side accumulation
     using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
     binary_op_type_host binary_op_host;
     using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
+    using cast_type = typename test_utils::select_plus_operator_host<T>::cast_type;
 
     // logical warp side for warp primitive, execution warp size is always rocprim::warp_size()
     static constexpr size_t logical_warp_size = TestFixture::params::warp_size;
@@ -335,12 +340,12 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ExclusiveScan)
     for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
     {
         unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
-        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+        SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
         // Generate data
         std::vector<T> input = test_utils::get_random_data<T>(size, 2, 50, seed_value);
         std::vector<T> output(size);
-        std::vector<T> expected(input.size(), (T)0);
+        std::vector<T> expected(input.size(), T(0));
         const T init = test_utils::get_random_value<T>(0, 100, seed_value);
 
         // Calculate expected results on host
@@ -352,7 +357,7 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ExclusiveScan)
             {
                 auto idx = i * logical_warp_size + j;
                 accumulator = binary_op_host(input[idx-1], accumulator);
-                expected[idx] = static_cast<T>(accumulator);
+                expected[idx] = static_cast<cast_type>(accumulator);
             }
         }
 
@@ -401,7 +406,7 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ExclusiveScan)
         );
 
         // Validating results
-        test_utils::assert_near(output, expected, test_utils::precision_threshold<T>::percentage);
+        test_utils::assert_near(output, expected, test_utils::precision<T> * logical_warp_size);
 
         HIP_CHECK(hipFree(device_input));
         HIP_CHECK(hipFree(device_output));
@@ -413,14 +418,15 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ExclusiveScan)
 typed_test_def(RocprimWarpScanTests, name_suffix, ExclusiveReduceScan)
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
-    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
     using T = typename TestFixture::params::type;
-     // for bfloat16 and half we use double for host-side accumulation
+    // for bfloat16 and half we use double for host-side accumulation
     using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
     binary_op_type_host binary_op_host;
     using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
+    using cast_type = typename test_utils::select_plus_operator_host<T>::cast_type;
 
     // logical warp side for warp primitive, execution warp size is always rocprim::warp_size()
     static constexpr size_t logical_warp_size = TestFixture::params::warp_size;
@@ -459,14 +465,14 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ExclusiveReduceScan)
     for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
     {
         unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
-        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+        SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
         // Generate data
         std::vector<T> input = test_utils::get_random_data<T>(size, 2, 50, seed_value);
         std::vector<T> output(size);
         std::vector<T> output_reductions(size / logical_warp_size);
-        std::vector<T> expected(input.size(), (T)0);
-        std::vector<T> expected_reductions(output_reductions.size(), (T)0);
+        std::vector<T> expected(input.size(), T(0));
+        std::vector<T> expected_reductions(output_reductions.size(), T(0));
         const T init = test_utils::get_random_value<T>(0, 100, seed_value);
 
         // Calculate expected results on host
@@ -478,7 +484,7 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ExclusiveReduceScan)
             {
                 auto idx = i * logical_warp_size + j;
                 accumulator = binary_op_host(input[idx-1], accumulator);
-                expected[idx] = static_cast<T>(accumulator);
+                expected[idx] = static_cast<cast_type>(accumulator);
             }
 
             acc_type accumulator_reductions(0);
@@ -486,7 +492,7 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ExclusiveReduceScan)
             {
                 auto idx = i * logical_warp_size + j;
                 accumulator_reductions = binary_op_host(input[idx], accumulator_reductions);
-                expected_reductions[i] = static_cast<T>(accumulator_reductions);
+                expected_reductions[i] = static_cast<cast_type>(accumulator_reductions);
             }
         }
 
@@ -549,8 +555,10 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ExclusiveReduceScan)
         );
 
         // Validating results
-        test_utils::assert_near(output, expected, test_utils::precision_threshold<T>::percentage);
-        test_utils::assert_near(output_reductions, expected_reductions, test_utils::precision_threshold<T>::percentage);
+        test_utils::assert_near(output, expected, test_utils::precision<T> * logical_warp_size);
+        test_utils::assert_near(output_reductions,
+                                expected_reductions,
+                                test_utils::precision<T> * logical_warp_size);
 
         HIP_CHECK(hipFree(device_input));
         HIP_CHECK(hipFree(device_output));
@@ -562,14 +570,15 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ExclusiveReduceScan)
 typed_test_def(RocprimWarpScanTests, name_suffix, Scan)
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
-    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
     using T = typename TestFixture::params::type;
-     // for bfloat16 and half we use double for host-side accumulation
+    // for bfloat16 and half we use double for host-side accumulation
     using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
     binary_op_type_host binary_op_host;
     using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
+    using cast_type = typename test_utils::select_plus_operator_host<T>::cast_type;
 
     // logical warp side for warp primitive, execution warp size is always rocprim::warp_size()
     static constexpr size_t logical_warp_size = TestFixture::params::warp_size;
@@ -608,14 +617,14 @@ typed_test_def(RocprimWarpScanTests, name_suffix, Scan)
     for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
     {
         unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
-        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+        SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
         // Generate data
         std::vector<T> input = test_utils::get_random_data<T>(size, 2, 50, seed_value);
         std::vector<T> output_inclusive(size);
         std::vector<T> output_exclusive(size);
-        std::vector<T> expected_inclusive(output_inclusive.size(), (T)0);
-        std::vector<T> expected_exclusive(output_exclusive.size(), (T)0);
+        std::vector<T> expected_inclusive(output_inclusive.size(), T(0));
+        std::vector<T> expected_exclusive(output_exclusive.size(), T(0));
         const T init = test_utils::get_random_value<T>(0, 100, seed_value);
 
         // Calculate expected results on host
@@ -628,11 +637,11 @@ typed_test_def(RocprimWarpScanTests, name_suffix, Scan)
             {
                 auto idx = i * logical_warp_size + j;
                 accumulator_inclusive = binary_op_host(input[idx], accumulator_inclusive);
-                expected_inclusive[idx] = static_cast<T>(accumulator_inclusive);
+                expected_inclusive[idx] = static_cast<cast_type>(accumulator_inclusive);
                 if(j > 0)
                 {
                     accumulator_exclusive = binary_op_host(input[idx-1], accumulator_exclusive);
-                    expected_exclusive[idx] = static_cast<T>(accumulator_exclusive);
+                    expected_exclusive[idx] = static_cast<cast_type>(accumulator_exclusive);
                 }
             }
         }
@@ -702,8 +711,12 @@ typed_test_def(RocprimWarpScanTests, name_suffix, Scan)
         );
 
         // Validating results
-        test_utils::assert_near(output_inclusive, expected_inclusive, test_utils::precision_threshold<T>::percentage);
-        test_utils::assert_near(output_exclusive, expected_exclusive, test_utils::precision_threshold<T>::percentage);
+        test_utils::assert_near(output_inclusive,
+                                expected_inclusive,
+                                test_utils::precision<T> * logical_warp_size);
+        test_utils::assert_near(output_exclusive,
+                                expected_exclusive,
+                                test_utils::precision<T> * logical_warp_size);
 
         HIP_CHECK(hipFree(device_input));
         HIP_CHECK(hipFree(device_inclusive_output));
@@ -715,14 +728,15 @@ typed_test_def(RocprimWarpScanTests, name_suffix, Scan)
 typed_test_def(RocprimWarpScanTests, name_suffix, ScanReduce)
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
-    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
     using T = typename TestFixture::params::type;
-     // for bfloat16 and half we use double for host-side accumulation
+    // for bfloat16 and half we use double for host-side accumulation
     using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
     binary_op_type_host binary_op_host;
     using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
+    using cast_type = typename test_utils::select_plus_operator_host<T>::cast_type;
 
     // logical warp side for warp primitive, execution warp size is always rocprim::warp_size()
     static constexpr size_t logical_warp_size = TestFixture::params::warp_size;
@@ -761,16 +775,16 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ScanReduce)
     for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
     {
         unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
-        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+        SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
         // Generate data
         std::vector<T> input = test_utils::get_random_data<T>(size, 2, 50, seed_value);
         std::vector<T> output_inclusive(size);
         std::vector<T> output_exclusive(size);
         std::vector<T> output_reductions(size / logical_warp_size);
-        std::vector<T> expected_inclusive(output_inclusive.size(), (T)0);
-        std::vector<T> expected_exclusive(output_exclusive.size(), (T)0);
-        std::vector<T> expected_reductions(output_reductions.size(), (T)0);
+        std::vector<T> expected_inclusive(output_inclusive.size(), T(0));
+        std::vector<T> expected_exclusive(output_exclusive.size(), T(0));
+        std::vector<T> expected_reductions(output_reductions.size(), T(0));
         const T init = test_utils::get_random_value<T>(0, 100, seed_value);
 
         // Calculate expected results on host
@@ -783,11 +797,11 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ScanReduce)
             {
                 auto idx = i * logical_warp_size + j;
                 accumulator_inclusive = binary_op_host(input[idx], accumulator_inclusive);
-                expected_inclusive[idx] = static_cast<T>(accumulator_inclusive);
+                expected_inclusive[idx] = static_cast<cast_type>(accumulator_inclusive);
                 if(j > 0)
                 {
                     accumulator_exclusive = binary_op_host(input[idx-1], accumulator_exclusive);
-                    expected_exclusive[idx] = static_cast<T>(accumulator_exclusive);
+                    expected_exclusive[idx] = static_cast<cast_type>(accumulator_exclusive);
                 }
             }
             expected_reductions[i] = expected_inclusive[(i+1) * logical_warp_size - 1];
@@ -875,9 +889,15 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ScanReduce)
         );
 
         // Validating results
-        test_utils::assert_near(output_inclusive, expected_inclusive, test_utils::precision_threshold<T>::percentage);
-        test_utils::assert_near(output_exclusive, expected_exclusive, test_utils::precision_threshold<T>::percentage);
-        test_utils::assert_near(output_reductions, expected_reductions, test_utils::precision_threshold<T>::percentage);
+        test_utils::assert_near(output_inclusive,
+                                expected_inclusive,
+                                test_utils::precision<T> * logical_warp_size);
+        test_utils::assert_near(output_exclusive,
+                                expected_exclusive,
+                                test_utils::precision<T> * logical_warp_size);
+        test_utils::assert_near(output_reductions,
+                                expected_reductions,
+                                test_utils::precision<T> * logical_warp_size);
 
         HIP_CHECK(hipFree(device_input));
         HIP_CHECK(hipFree(device_inclusive_output));
@@ -889,12 +909,13 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ScanReduce)
 typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScanCustomType)
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
-    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
     using base_type = typename TestFixture::params::type;
     using T = test_utils::custom_test_type<base_type>;
     using acc_type = typename test_utils::select_plus_operator_host<base_type>::acc_type;
+    using cast_type = typename test_utils::select_plus_operator_host<base_type>::cast_type;
 
     // logical warp side for warp primitive, execution warp size is always rocprim::warp_size()
     static constexpr size_t logical_warp_size = TestFixture::params::warp_size;
@@ -933,7 +954,7 @@ typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScanCustomType)
     for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
     {
         unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
-        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+        SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
         // Generate data
         std::vector<T> input(size);
@@ -958,7 +979,7 @@ typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScanCustomType)
             {
                 auto idx = i * logical_warp_size + j;
                 accumulator = static_cast<test_utils::custom_test_type<acc_type>>(input[idx]) + accumulator;
-                expected[idx] = static_cast<T>(accumulator);
+                expected[idx] = static_cast<test_utils::custom_test_type<cast_type>>(accumulator);
             }
         }
 
@@ -1007,7 +1028,9 @@ typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScanCustomType)
         );
 
         // Validating results
-        test_utils::assert_near(output, expected, test_utils::precision_threshold<base_type>::percentage);
+        test_utils::assert_near(output,
+                                expected,
+                                test_utils::precision<base_type> * logical_warp_size);
 
         HIP_CHECK(hipFree(device_input));
         HIP_CHECK(hipFree(device_output));
